@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2017 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2018 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -26,6 +26,7 @@
 
 #include "core/siril.h"
 #include "core/processing.h"
+#include "gui/progress_and_log.h"
 #include "gui/callbacks.h"
 #include "io/single_image.h"
 #include "gui/histogram.h"
@@ -389,6 +390,7 @@ gboolean end_enhance_saturation(gpointer p) {
 	adjust_cutoff_from_updated_gfit();
 	redraw(com.cvport, REMAP_ALL);
 	redraw_previews();
+	update_gfit_histogram_if_needed();
 	free(args);
 	set_cursor_waiting(FALSE);
 	update_used_memory();
@@ -476,6 +478,7 @@ gboolean end_scnr(gpointer p) {
 	adjust_cutoff_from_updated_gfit();
 	redraw(com.cvport, REMAP_ALL);
 	redraw_previews();
+	update_gfit_histogram_if_needed();
 	free(args);
 	set_cursor_waiting(FALSE);
 	update_used_memory();
@@ -691,6 +694,7 @@ void on_button_bkg_neutralization_clicked(GtkButton *button, gpointer user_data)
 
 	redraw(com.cvport, REMAP_ALL);
 	redraw_previews();
+	update_gfit_histogram_if_needed();
 	set_cursor_waiting(FALSE);
 }
 
@@ -908,6 +912,7 @@ void on_calibration_apply_button_clicked(GtkButton *button, gpointer user_data)
 
 	redraw(com.cvport, REMAP_ALL);
 	redraw_previews();
+	update_gfit_histogram_if_needed();
 	set_cursor_waiting(FALSE);
 }
 
