@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2017 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2018 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -33,12 +33,11 @@
 #include "io/single_image.h"
 #include "gui/PSF_list.h"
 #include "gui/callbacks.h"
+#include "gui/progress_and_log.h"
 #include "registration/registration.h"
 #include "compositing/filters.h"
 #include "stacking/stacking.h"
-#ifdef HAVE_OPENCV
 #include "opencv/opencv.h"
-#endif
 
 #undef DEBUG
 
@@ -459,7 +458,6 @@ void on_filechooser_file_set(GtkFileChooserButton *widget, gpointer user_data) {
 			 * in, or even taken from fit->binning_x and binning_y */
 			//if (layers[layer]->the_fit.binning_x > 1 ||
 			//layers[layer]->the_fit.binning_y > 1)
-#ifdef HAVE_OPENCV
 			if (gfit.rx < layers[layer]->the_fit.rx ||
 					gfit.ry < layers[layer]->the_fit.ry) {
 				siril_log_message(_("The first loaded image should have the greatest sizes for now\n"));
@@ -476,12 +474,6 @@ void on_filechooser_file_set(GtkFileChooserButton *widget, gpointer user_data) {
 				image_find_minmax(&layers[layer]->the_fit, 0);
 				gtk_label_set_text(layers[layer]->label, buf);
 			}
-#else
-			siril_log_message(_("You need to install opencv to compose images with different sizes\n"));
-			sprintf(buf, _("NOT OK %ux%u"), layers[layer]->the_fit.rx, layers[layer]->the_fit.ry);
-			gtk_label_set_text(layers[layer]->label, buf);
-			retval = 1;
-#endif
 		}
 		else if (!retval) {
 			image_find_minmax(&layers[layer]->the_fit, 0);
@@ -518,7 +510,7 @@ void on_filechooser_file_set(GtkFileChooserButton *widget, gpointer user_data) {
 		clear_stars_list();
 		com.seq.current = UNRELATED_IMAGE;
 		com.uniq = calloc(1, sizeof(single));
-		com.uniq->comment = strdup("Compositing result image");
+		com.uniq->comment = strdup(_("Compositing result image"));
 		com.uniq->filename = strdup(_("Unsaved compositing result"));
 		com.uniq->nb_layers = gfit.naxes[2];
 		com.uniq->layers = calloc(com.uniq->nb_layers, sizeof(layer_info));
@@ -579,7 +571,7 @@ void create_the_internal_sequence() {
 	seq->ry = gfit.ry;
 }
 
-/* start alignming the layers: create an 'internal' sequence and run the selected method on it */
+/* start aligning the layers: create an 'internal' sequence and run the selected method on it */
 void on_button_align_clicked(GtkButton *button, gpointer user_data) {
 	int i, j;
 	struct registration_args regargs;
@@ -608,21 +600,22 @@ void on_button_align_clicked(GtkButton *button, gpointer user_data) {
 		set_progress_bar_data(_("Error in layers alignment."), PROGRESS_DONE);
 	else set_progress_bar_data(_("Registration complete."), PROGRESS_DONE);
 	set_cursor_waiting(FALSE);
+	com.run_thread = FALSE;	// fix for the cancelling check in processing
 
 	/* display the values */
 	if (luminance_mode) {
 		for (j=0, i=0; i<layers_count; i++) {
 			if (has_fit(i)) {
-				gtk_spin_button_set_value(layers[i]->spinbutton_x, seq->regparam[0][j].shiftx);
-				gtk_spin_button_set_value(layers[i]->spinbutton_y, seq->regparam[0][j].shifty);
+				gtk_spin_button_set_value(layers[i]->spinbutton_x, roundf_to_int(seq->regparam[0][j].shiftx));
+				gtk_spin_button_set_value(layers[i]->spinbutton_y, roundf_to_int(seq->regparam[0][j].shifty));
 				j++;
 			}
 		}
 	} else {
 		for (j=0, i=1; i<layers_count; i++) {
 			if (has_fit(i)) {
-				gtk_spin_button_set_value(layers[i]->spinbutton_x, seq->regparam[0][j].shiftx);
-				gtk_spin_button_set_value(layers[i]->spinbutton_y, seq->regparam[0][j].shifty);
+				gtk_spin_button_set_value(layers[i]->spinbutton_x, roundf_to_int(seq->regparam[0][j].shiftx));
+				gtk_spin_button_set_value(layers[i]->spinbutton_y, roundf_to_int(seq->regparam[0][j].shifty));
 				j++;
 			}
 		}
@@ -649,9 +642,9 @@ WORD get_composition_pixel_value(int fits_index, int reg_layer, int x, int y) {
 	int realX = x, realY = y;
 	if (seq && seq->regparam && reg_layer < seq->number && reg_layer >= 0) {
 		// all images have one layer, hence the 0 below
-		realX = x - seq->regparam[0][reg_layer].shiftx;
+		realX = x - roundf_to_int(seq->regparam[0][reg_layer].shiftx);
 		if (realX < 0 || realX >= gfit.rx) return (WORD)0;
-		realY = y - seq->regparam[0][reg_layer].shifty;
+		realY = y - roundf_to_int(seq->regparam[0][reg_layer].shifty);
 		if (realY < 0 || realY >= gfit.ry) return (WORD)0;
 	}
 	WORD pixel_value = layers[fits_index]->the_fit.pdata[0][realX + realY * gfit.rx];
@@ -707,7 +700,7 @@ void update_compositing_interface() {
 		gtk_label_set_text(label, "");
 		gtk_widget_set_sensitive(lookup_widget("button_align"), TRUE);
 	}
-
+	update_MenuItem();
 }
 
 /* callback for changes of the selected reference layer */
