@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2017 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2018 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -45,6 +45,7 @@
 #include "gui/PSF_list.h"
 #include "gui/histogram.h"
 #include "gui/plot.h"
+#include "gui/progress_and_log.h"
 #include "algos/colors.h"
 #include "algos/PSF.h"
 #include "algos/star_finder.h"
@@ -55,13 +56,11 @@
 #include "algos/quality.h"
 #include "algos/cosmetic_correction.h"
 #include "stacking/stacking.h"
+#include "stacking/sum.h"
 #include "registration/registration.h"
-
-#ifdef HAVE_OPENCV
 #include "opencv/opencv.h"
-#endif
 
-static char *word[MAX_COMMAND_WORDS];
+static char *word[MAX_COMMAND_WORDS];	// NULL terminated
 
 command commande[] = {
 	/* name,	nbarg,	usage,			function pointer */
@@ -75,7 +74,7 @@ command commande[] = {
 	{"clearstar", 0, "clearstar", process_clearstar},
 	{"contrast", 0, "contrast", process_contrast},
 	{"cosme", 1, "cosme [filename].lst", process_cosme},
-	{"cosme_cfa", 1, "cosme [filename].lst", process_cosme},
+	{"cosme_cfa", 1, "cosme_cfa [filename].lst", process_cosme},
 	{"crop", 0, "crop [x y width height]", process_crop}, 
 
 	{"ddp", 3, "ddp level coef sigma", process_ddp}, 
@@ -115,7 +114,7 @@ command commande[] = {
 	// extension to a higher priority in case two files with same basename
 	// exist (stat_file() manages that priority order for now).
 	{"log", 0, "log ", process_log}, /* logarifies current image */
-#ifndef WIN32
+#ifndef _WIN32
 	{"ls", 0, "ls ", process_ls},
 #endif
 	
@@ -129,13 +128,10 @@ command commande[] = {
 	
 	{"psf", 0, "psf", process_psf},
 	
-#ifdef HAVE_OPENCV
 	{"resample", 1, "resample factor", process_resample},
-#endif	
+	{"rl", 2, "rl iterations sigma", process_rl},
 	{"rmgreen", 1, "rmgreen type", process_scnr},
-#ifdef HAVE_OPENCV
 	{"rotate", 1, "rotate angle", process_rotate},
-#endif
 	{"rotatePi", 0, "rotatePi", process_rotatepi},
 	
 	{"satu", 1, "satu coeff ", process_satu}, 
@@ -143,6 +139,9 @@ command commande[] = {
 	{"savebmp", 1, "savebmp filename (save display image in bmp)", process_savebmp}, 
 #ifdef HAVE_LIBJPEG
 	{"savejpg", 1, "savejpg filename [quality] (save current display in jpg)", process_savejpg},
+#endif
+#ifdef HAVE_LIBPNG
+	{"savepng", 1, "savepng filename (save current display in png)", process_savepng},
 #endif
 	{"savepnm", 1, "savepnm filename (save current image in Netpbm)", process_savepnm},
 #ifdef HAVE_LIBTIFF
@@ -266,6 +265,19 @@ int process_savejpg(int nb){
 }
 #endif
 
+#ifdef HAVE_LIBPNG
+int process_savepng(int nb){
+	char filename[256];
+
+	strcpy(filename, word[1]);
+	strcat(filename, ".png");
+	set_cursor_waiting(TRUE);
+	savepng(filename, &gfit, 2, gfit.naxes[2] == 3);
+	set_cursor_waiting(FALSE);
+	return 0;
+}
+#endif
+
 #ifdef HAVE_LIBTIFF
 int process_savetif(int nb){
 	char filename[256];
@@ -281,25 +293,7 @@ int process_savetif(int nb){
 #endif
 
 int process_savepnm(int nb){
-	char filename[256];
-	
-	switch(gfit.naxes[2]){
-		case 1:
-			sprintf(filename,"%s", strcat(word[1],".pgm"));
-				set_cursor_waiting(TRUE);
-				savepgm(filename, &(gfit));
-				set_cursor_waiting(FALSE);
-		break;
-		case 3:
-			sprintf(filename,"%s", strcat(word[1],".ppm"));
-			set_cursor_waiting(TRUE);
-			saveppm(filename, &(gfit));
-			set_cursor_waiting(FALSE);
-		break;
-		
-		default:	/*Should not happen */
-			return 1;
-		}
+	saveNetPBM(word[1], &gfit);
 	return 0;	
 }
 
@@ -376,6 +370,43 @@ int process_gauss(int nb){
 	return 0;
 }
 
+int process_rl(int nb) {
+
+	double sigma;
+	int iter;
+
+	control_window_switch_to_tab(OUTPUT_LOGS);
+	iter = atoi(word[1]);
+	sigma = atof(word[2]);
+	if (iter <= 0) {
+		siril_log_message(_("Number of iterations must be > 0.\n"));
+		return 1;
+	}
+
+	if (sigma <= 0) {
+		siril_log_message(_("Sigma must be > 0.\n"));
+		return 1;
+	}
+
+	if (get_thread_run()) {
+		siril_log_message(
+				_("Another task is already in progress, ignoring new request.\n"));
+		return 1;
+	}
+
+	struct RL_data *args = malloc(sizeof(struct RL_data));
+
+	set_cursor_waiting(TRUE);
+
+	args->fit = &gfit;
+	args->sigma = sigma;
+	args->iter = iter;
+
+	start_in_new_thread(LRdeconv, args);
+
+	return 0;
+}
+
 int process_unsharp(int nb){
 	unsharp(&(gfit), atof(word[1]), atof(word[2]), TRUE);
 	adjust_cutoff_from_updated_gfit();
@@ -422,42 +453,39 @@ int process_crop(int nb){
 	return 0;
 }
 
-int process_cd(int nb){
+int process_cd(int nb) {
 	char filename[256];
-	int retval, i;
-	
-	strncpy(filename, word[1], 250);
-	filename[250] = '\0';
-	
-	for (i=1; i<nb-1; ++i){
-		strcat(filename, " ");
-		strcat(filename, word[i+1]);
-	}
-	
+	int retval;
+
+	g_strlcpy(filename, word[1], 250);
+
 	expand_home_in_filename(filename, 256);
-	if (!(retval=changedir(filename)))
+	retval = changedir(filename, NULL);
+	if (!retval) {
 		writeinitfile();
+	}
 	return retval;
 }
 
-int process_wrecons(int nb){
+int process_wrecons(int nb) {
 	int i;
 	float coef[7];
-	char *File_Name_Transform[3] = {"r_rawdata.wave", "g_rawdata.wave", "b_rawdata.wave"}, *dir[3];
+	char *File_Name_Transform[3] = { "r_rawdata.wave", "g_rawdata.wave",
+			"b_rawdata.wave" }, *dir[3];
 	const char *tmpdir;
 	int nb_chan = gfit.naxes[2];
-	
+
 	assert(nb_chan == 1 || nb_chan == 3);
-		
- 	tmpdir = g_get_tmp_dir();
- 	
-	for (i=0;i<nb-1;++i){
-		coef[i] = atof(word[i+1]);
+
+	tmpdir = g_get_tmp_dir();
+
+	for (i = 0; i < nb - 1; ++i) {
+		coef[i] = atof(word[i + 1]);
 	}
 
-	for (i=0; i < nb_chan; i++) {
+	for (i = 0; i < nb_chan; i++) {
 		dir[i] = g_build_filename(tmpdir, File_Name_Transform[i], NULL);
-		wavelet_reconstruct_file (dir[i], coef, gfit.pdata[i]);
+		wavelet_reconstruct_file(dir[i], coef, gfit.pdata[i]);
 		g_free(dir[i]);
 	}
 
@@ -515,7 +543,7 @@ int process_log(int nb){
 	return 0;
 }
 
-#ifndef WIN32
+#ifndef _WIN32
 int process_ls(int nb){
 	struct dirent **list;
 	gchar *path = NULL;
@@ -527,7 +555,7 @@ int process_ls(int nb){
 			if (word[1][0] == G_DIR_SEPARATOR || word[1][0] == '~') {
 				char filename[256];
 				
-				strncpy(filename, word[1], 250);
+				g_strlcpy(filename, word[1], 250);
 				filename[250] = '\0';
 				expand_home_in_filename(filename, 256);
 				path = g_build_filename(filename, NULL);
@@ -549,7 +577,7 @@ int process_ls(int nb){
 			siril_log_message(_("Cannot list files, set working directory first.\n"));
 			return 1;
 		}
-		path = strdup(com.wd);
+		path = g_strdup(com.wd);
 	}
 	if (path == NULL) {
 		siril_log_message(_("Siril cannot open the directory.\n"));
@@ -576,7 +604,7 @@ int process_ls(int nb){
 
 		filename = g_build_filename(path, list[i]->d_name, NULL);
 
-		if (lstat(filename, &entrystat)) {
+		if (g_lstat(filename, &entrystat)) {
 			perror("stat");
 			g_free(filename);
 			break;
@@ -630,7 +658,6 @@ int	process_mirrory(int nb){
 	return 0;
 }
 
-#ifdef HAVE_OPENCV
 int process_resample(int nb) {
 	double factor = atof(word[1]);
 	if (factor > 5.0) {
@@ -662,14 +689,10 @@ int process_rotate(int nb) {
 	set_cursor_waiting(FALSE);
 	return 0;
 }
-#endif
 
 int process_rotatepi(int nb){
-#ifdef HAVE_OPENCV
 	verbose_rotate_image(&gfit, 180.0, OPENCV_LINEAR, 1);
-#else
-	fits_rotate_pi(&gfit);
-#endif
+
 	redraw(com.cvport, REMAP_ALL);
 	redraw_previews();
 	return 0;	
@@ -851,7 +874,7 @@ int process_histo(int nb){
 	else clayer = vport_number_to_name(nlayer);
 	snprintf(name, 20, "histo_%s.dat",clayer);
 
-	FILE *f = fopen(name, "w");
+	FILE *f = g_fopen(name, "w");
 
 	if (f == NULL) {
 		free(clayer);
@@ -940,7 +963,10 @@ int process_new(int nb){
 
 	close_single_image();
 
-	new_fit_image(&gfit, width, height, layers);
+	fits *fit = &gfit;
+	if (new_fit_image(&fit, width, height, layers))
+		return 1;
+	memset(gfit.data, 0, width*height*layers*sizeof(WORD));
 
 	open_single_image_from_gfit(strdup(_("new empty image")));
 	return 0;
@@ -1015,7 +1041,7 @@ int process_findhot(int nb){
 	siril_log_message(_("%ld cold and %ld hot pixels\n"), icold, ihot);
 
 	sprintf(filename, "%s.lst", word[1]);
-	FILE *cosme_file = fopen(filename, "w");
+	FILE *cosme_file = g_fopen(filename, "w");
 	if (cosme_file == NULL) {
 		siril_log_message(_("Cannot open file: %s\n"), filename);
 		free(dev);
@@ -1048,7 +1074,7 @@ int process_cosme(int nb) {
 
 	if (!ends_with(word[1], ".lst"))
 		strcat(word[1], ".lst");
-	cosme_file = fopen(word[1], "r");
+	cosme_file = g_fopen(word[1], "r");
 	if (cosme_file == NULL) {
 		siril_log_message(_("Cannot open file: %s\n"), word[1]);
 		return 1;
@@ -1095,7 +1121,6 @@ int process_cosme(int nb) {
 			cosmeticCorrOneLine(&gfit, dev, is_cfa);
 			break;
 		case 'C':
-#ifdef HAVE_OPENCV
 			nb_tokens = sscanf(line + 2, "%lf %lf %c", &dev.p.y, &dirty, &type);
 			if (nb_tokens != 2 && nb_tokens != 3) {
 				fprintf(stderr, "cosmetic correction: "
@@ -1108,10 +1133,7 @@ int process_cosme(int nb) {
 			cvRotateImage(&gfit, 90.0, -1, OPENCV_LINEAR);
 			cosmeticCorrOneLine(&gfit, dev, is_cfa);
 			cvRotateImage(&gfit, -90.0, -1, OPENCV_LINEAR);
-#else
-			siril_log_message(_("Opencv need to be compiled to remove bad column.\n"));
-			retval = 1;
-#endif
+
 			break;
 		default:
 			fprintf(stderr, _("cosmetic correction: "
@@ -1401,57 +1423,121 @@ int process_stat(int nb){
 	return 0;
 }
 
-gpointer stackall_worker(gpointer args) {
-	DIR *dir;
-	struct dirent *file;
-	int number_of_loaded_sequences = 0, retval = 0;
+struct _stackall_data {
+	stack_method method;
+	double sig[2];
+	gboolean force_no_norm;
+};
 
+gpointer stackall_worker(gpointer garg) {
+	GDir *dir;
+	GError *error = NULL;
+	const gchar *file;
+	int number_of_loaded_sequences = 0, retval = 0;
+	struct _stackall_data *arg = (struct _stackall_data *)garg;
+
+	control_window_switch_to_tab(OUTPUT_LOGS);
 	siril_log_message(_("Looking for sequences in current working directory...\n"));
-	if (check_seq(0) || (dir = opendir(com.wd)) == NULL) {
+	if (check_seq(0) || (dir = g_dir_open(com.wd, 0, &error)) == NULL) {
 		siril_log_message(_("Error while searching sequences or opening the directory.\n"));
+		fprintf (stderr, "stackall: %s\n", error->message);
 		com.wd[0] = '\0';
 		gdk_threads_add_idle(end_generic, NULL);
 		return NULL;
 	}
 	siril_log_message(_("Starting stacking of found sequences...\n"));
-	while ((file = readdir(dir)) != NULL) {
+	while ((file = g_dir_read_name(dir)) != NULL) {
 		char *suf;
 
-		if ((suf = strstr(file->d_name, ".seq")) && strlen(suf) == 4) {
-			sequence *seq = readseqfile(file->d_name);
+		if ((suf = strstr(file, ".seq")) && strlen(suf) == 4) {
+			sequence *seq = readseqfile(file);
 			if (seq != NULL) {
 				char filename[256];
 				struct stacking_args args;
-				//args.method = stack_summing;
+				if (seq_check_basic_data(seq, FALSE) == -1) {
+					free(seq);
+					continue;
+				}
+				siril_log_message(_("Stacking sequence %s\n"), seq->seqname);
 				args.seq = seq;
 				args.filtering_criterion = stack_filter_all;
+				args.filtering_parameter = 0.0;
 				args.nb_images_to_stack = seq->number;
-				snprintf(filename, 256, "%s%sstacked%s", seq->seqname,
-						ends_with(seq->seqname, "_") ?
-								"" : (ends_with(com.seq.seqname, "-") ? "" : "_"),
-						com.ext);
+				args.image_indices = malloc(seq->number * sizeof(int));
 				gettimeofday(&args.t_start, NULL);
+				args.max_number_of_rows = stack_get_max_number_of_rows(seq, seq->number);
+				// the three below: used only if method is average w/ rejection
+				args.sig[0] = arg->sig[0];
+				args.sig[1] = arg->sig[1];
+				args.type_of_rejection = WINSORIZED;
+				if (!arg->force_no_norm &&
+						(arg->method == stack_median || arg->method == stack_mean_with_rejection))
+					args.normalize = ADDITIVE_SCALING;
+				else args.normalize = NO_NORM;
+				args.force_norm = FALSE;
+				args.reglayer = get_registration_layer();
+				stack_fill_list_of_unfiltered_images(&args);
 
-				retval = stack_summing(&args);
-				if (savefits(filename, &gfit))
-					siril_log_message(_("Could not save the stacking result %s\n"),
-							filename);
+				char *suffix = ends_with(seq->seqname, "_") ? "" :
+				       	(ends_with(com.seq.seqname, "-") ? "" : "_");
+				snprintf(filename, 256, "%s%sstacked%s",
+						seq->seqname, suffix, com.ext);
+
+				retval = arg->method(&args);
 
 				free_sequence(seq, TRUE);
-				++number_of_loaded_sequences;
-				if (retval) break;
+				free(args.image_indices);
+				if (!retval) {
+					if (savefits(filename, &gfit))
+						siril_log_color_message(_("Could not save the stacking result %s\n"),
+								"red", filename);
+					++number_of_loaded_sequences;
+				}
+				else if (!get_thread_run()) break;
+
 			}
 		}
 	}
-	closedir(dir);
-	siril_log_message(_("Stacked %d sequences %s.\n"), number_of_loaded_sequences,
-			retval ? _("with errors") : _("successfully"));
+	g_dir_close(dir);
+	free(arg);
+	siril_log_message(_("Stacked %d sequences successfully.\n"), number_of_loaded_sequences);
 	gdk_threads_add_idle(end_generic, NULL);
 	return NULL;
 }
 
 int process_stackall(int nb) {
-	start_in_new_thread(stackall_worker, NULL);
+	struct _stackall_data *arg = malloc(sizeof (struct _stackall_data));
+	arg->force_no_norm = FALSE;
+
+	if (!word[1] || !strcmp(word[1], "sum"))
+		arg->method = stack_summing_generic;
+	else if (!strcmp(word[1], "max"))
+		arg->method = stack_addmax;
+	else if (!strcmp(word[1], "min"))
+		arg->method = stack_addmin;
+	else if (!strcmp(word[1], "med") || !strcmp(word[1], "median")) {
+		arg->method = stack_median;
+		if (word[2] && (!strcmp(word[2], "-nonorm") || !strcmp(word[2], "-no_norm")))
+			arg->force_no_norm = TRUE;
+	} else if (!strcmp(word[1], "rej") || !strcmp(word[1], "mean")) {
+		if (!word[2] || !word[3] ||
+				(arg->sig[0] = atof(word[2])) < 0.001 ||
+				(arg->sig[1] = atof(word[3])) < 0.001) {
+			siril_log_message(_("The average stacking with rejection uses the Winsorized rejection here and requires two extra arguments: sigma low and high.\n"));
+			free(arg);
+			return 1;
+		}
+		arg->method = stack_mean_with_rejection;
+		if (word[4] && (!strcmp(word[4], "-nonorm") || !strcmp(word[4], "-no_norm")))
+			arg->force_no_norm = TRUE;
+	}
+	else {
+		siril_log_message(_("The provided type of stacking is unknown (%s).\n"), word[1]);
+		free(arg);
+		return 1;
+	}
+
+	start_in_new_thread(stackall_worker, arg);
 	return 0;
 }
 
@@ -1588,7 +1674,7 @@ int processcommand(const char *line) {
 		FILE * fp;
 
 
-		fp = fopen(line + 1, "r");
+		fp = g_fopen(line + 1, "r");
 		if (fp == NULL) {
 			siril_log_message(_("File [%s] does not exist\n"), line + 1);
 			return 1;
