@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2017 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2018 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -45,17 +45,18 @@
 #include "core/proto.h"
 #include "core/processing.h"
 #include "gui/callbacks.h"
-#include "algos/colors.h"
 #include "gui/histogram.h"
+#include "gui/progress_and_log.h"
+#include "gui/PSF_list.h"
 #include "io/sequence.h"
 #include "io/single_image.h"
+#include "io/ser.h"
 #include "algos/gradient.h"
-#include "gui/PSF_list.h"
-#include "opencv/opencv.h"
+#include "algos/colors.h"
 #include "algos/Def_Math.h"
 #include "algos/Def_Wavelet.h"
 #include "algos/cosmetic_correction.h"
-#include "io/ser.h"
+#include "opencv/opencv.h"
 
 #define MAX_ITER 15
 #define EPSILON 1E-4
@@ -190,10 +191,11 @@ int imoper(fits *a, fits *b, char oper) {
  * it will use double type format.
  */
 int sub_background(fits* image, fits* background, int layer) {
-	double *pxl_image, *pxl_bkg, min = DBL_MAX;
+	double *pxl_image, *pxl_bkg;
 	WORD *image_buf = image->pdata[layer];
 	WORD *bkg_buf = background->pdata[layer];
 	size_t i, ndata;
+	double median;
 
 	if ((image->rx) != (background->rx) || ((image->ry) != (background->ry))) {
 		char *msg = siril_log_message(
@@ -204,23 +206,24 @@ int sub_background(fits* image, fits* background, int layer) {
 	}
 	ndata = image->rx * image->ry;
 
-	// First step we convert data, apply the subtraction and search for minimum
+	/* First step we convert data, apply the subtraction, normalize with median,
+	 * and re-convert data to USHORT
+	 */
+	imstats *stat = statistics(image, layer, NULL, STATS_BASIC, STATS_ZERO_NULLCHECK);
+	median = (double) stat->median / USHRT_MAX_DOUBLE;
 	pxl_image = malloc(sizeof(double) * ndata);
 	pxl_bkg = malloc(sizeof(double) * ndata);
+
 	for (i = 0; i < ndata; i++) {
 		pxl_image[i] = (double) image_buf[i] / USHRT_MAX_DOUBLE;
 		pxl_bkg[i] = (double) bkg_buf[i] / USHRT_MAX_DOUBLE;
 		pxl_image[i] -= pxl_bkg[i];
-		min = min(pxl_image[i], min);
-	}
-	image_buf = image->pdata[layer];
-	// Second we apply an offset to the result and re-convert the data
-	for (i = 0; i < ndata; i++) {
-		pxl_image[i] += fabs(min);
+		pxl_image[i] += median;
 		image_buf[i] = round_to_WORD(pxl_image[i] * USHRT_MAX_DOUBLE);
 	}
 
 	// We free memory
+	free(stat);
 	free(pxl_image);
 	free(pxl_bkg);
 	return 0;
@@ -312,7 +315,6 @@ int ndiv(fits *a, fits *b) {
  *
  */
 
-#ifdef HAVE_OPENCV
 int unsharp(fits *fit, double sigma, double amount, gboolean verbose) {
 	struct timeval t_start, t_end;
 
@@ -331,103 +333,6 @@ int unsharp(fits *fit, double sigma, double amount, gboolean verbose) {
 	return 0;
 }
 
-#else
-double gaussienne(double sigma, int size, double *gauss) {
-	double s2, s;
-	int i,j,n;
-
-	n=size/2;
-	s2=sigma*sigma;
-	s=(double)0;
-	for (i=0;i<size;++i) {
-		for (j=0;j<size;++j) {
-			s+=gauss[i*size+j]=exp(-((double)(i-n)*(i-n) + (double)(j-n)*(j-n))/2/s2);
-			//~ fprintf(stderr,"%d:%d %f %f \n", i, j, s, gauss[i*size+j]);
-		}
-	}
-	return s;
-}
-
-int unsharp(fits *fit, double sigma, double mult, gboolean verbose) {
-	// if fabs(mult) > 0.01, unsharp computes the unsharp mask
-	// else unsharp computes a gaussian filter
-
-	double normalize,g, *gauss/*, *gaussbuf*/;
-	WORD *buf, *gbuf;
-	int size, ss2, stride, i, j, k, l, layer;
-	struct timeval t_start, t_end;
-
-	if (verbose) {
-		siril_log_color_message("Unsharp: processing...\n", "red");
-		gettimeofday (&t_start, NULL);
-	}
-
-	//	fprintf(stderr,"gfitrx: %d mult :%f\n", gfit.rx, mult);
-	//~ size=(int)(4*sigma);
-	size = (int)(2*(((sigma - 0.8) / 0.3) + 1));// heuristic. Just to be homogeneous with opencv
-	if (!(size%2))
-	++size;
-
-	ss2=size/2;
-	gauss=malloc(size*size*sizeof(double));
-	if (!gauss) {
-		perror("unsharp.c: Alloc error for gauss");
-		return -1;
-	}
-	//	fprintf(stderr,"size: %d sigma:%f\n", size, sigma);
-
-	normalize=gaussienne(sigma, size, gauss);
-	//	fprintf(stderr,"gfitrx: %d ss2:%d\n", gfit.rx, ss2);
-	wfit[4].data=(WORD *)calloc(1, fit->rx*fit->ry*sizeof(WORD));
-
-	wfit[4].rx=fit->rx;
-	wfit[4].ry=fit->ry;
-	wfit[4].lo=fit->lo;
-	wfit[4].hi=fit->hi;
-	stride=wfit[4].rx-size;
-	for (layer=0; layer<fit->naxes[2]; ++layer) {
-		memcpy(wfit[4].data, fit->pdata[layer], fit->rx*fit->ry*sizeof(WORD));
-
-		buf=fit->pdata[layer]+ss2+ss2*fit->rx;
-		//	fprintf(stderr,"gfitrx: %d ss2:%d\n", gfit.rx, ss2);
-		for (i=ss2;i<fit->ry-ss2;++i) {
-			for (j=ss2;j<fit->rx-ss2;++j) {
-				g=(double)0;
-				gbuf=wfit[4].data+(i-ss2)*fit->rx+j-ss2;
-				//~ gaussbuf=gauss;
-				for (k=0;k<size;++k) {
-					for(l=0;l<size;++l) {
-						g+=(*gbuf++)*(gauss[k*size+l]);
-					}
-					gbuf+=stride;
-				}
-				*(buf++)=g/normalize;
-			}
-			buf+=ss2+ss2;
-		}
-
-		buf=fit->pdata[layer];
-		gbuf=wfit[4].data;
-		if (fabs(mult)>0.0) {
-			for (i=0; i<fit->rx * fit->ry; i++) {
-				//~ double tmp = gbuf[i] + mult * (gbuf[i] - buf[i]);
-				double tmp = gbuf[i] * (1.0 + mult) + buf[i] * (- mult);
-				if (tmp < 0.0) buf[i] = 0;
-				else if (tmp > USHRT_MAX_DOUBLE) buf[i] = USHRT_MAX;
-				else buf[i] = (WORD) tmp;
-			}
-		}
-	}
-	free(gauss);
-	clearfits(&wfit[4]);
-	if (verbose) {
-		gettimeofday (&t_end, NULL);
-		show_time(t_start, t_end);
-	}
-	return 0;
-}
-#endif
-
 /* inplace cropping of the image in fit
  * fit->data is not realloc, only fit->pdata points to a different area and
  * data is correctly written to this new area, which makes this function
@@ -509,58 +414,6 @@ int shift(int sx, int sy) {
 	return 0;
 }
 
-#if 0
-int rshift2(char *genname, char *outname, int number, char *shiftfile) {
-	int j,shiftx,shifty,count,n;
-	char line[256];
-	FILE *sf=NULL;
-
-	if (shiftfile!=NULL) {
-		sf=fopen(shiftfile,"r");
-		if (sf==NULL) {
-			siril_log_message("rshift2: could not open shift file %s\n", shiftfile);
-			return 0;
-		}
-		fgets(line,255,sf);
-		while(line[0]=='#') {
-			fgets(line,255,sf);
-		}
-	}
-	shiftx=shifty=0;
-
-	for (j=1;j<=number;++j) {
-		if(sf!=NULL) {
-			count=sscanf(line,"%d %d %d",&n,&shiftx,&shifty);
-			if(count!=3) {
-				siril_log_message("rshift2: format error in shift file %s\n", shiftfile);
-				return 0;
-			}
-			fgets(line,255,sf);
-		}
-		else {
-			shiftx=0;
-			shifty=0;
-			n=j;
-		}
-		fprintf(stderr,"rshift2 %d %d %d\n", n, shiftx,shifty);
-		siril_log_message("Processing image %s%d%s", genname, n, com.ext);
-
-		buildfilename(genname,n);
-		if(readfits(com.formname, &(gfit), com.formname)) {
-			fprintf(stderr,"missed %s com.formanme",com.formname);
-			return 1;
-		}
-		shift(shiftx, shifty);
-		buildfilename(outname,n);
-		savefits(com.formname,&gfit);
-	}
-	if(sf!=NULL) {
-		fclose(sf);
-	}
-	return 0;
-}
-#endif
-
 /* This entropy function computes the entropy for the image in gfit for its
  * layer 'layer', in the area designated by area which can be NULL.
  * An optional imstats parameter can be used to provide the background and
@@ -885,15 +738,16 @@ int lrgb(fits *l, fits *r, fits *g, fits *b, fits *lrgb) {
 
 static double evaluateNoiseOfCalibratedImage(fits *fit, fits *dark, double k) {
 	double noise = 0;
-	fits *dark_tmp;
-	fits *fit_tmp;
+	fits *dark_tmp = NULL, *fit_tmp = NULL;
 	int chan;
 
-	dark_tmp = calloc(1, sizeof(fits));
-	fit_tmp = calloc(1, sizeof(fits));
-
-	new_fit_image(dark_tmp, dark->rx, dark->ry, 1);
-	new_fit_image(fit_tmp, fit->rx, fit->ry, 1);
+	if (new_fit_image(&dark_tmp, dark->rx, dark->ry, 1)) {
+		return -1.0;
+	}
+	if (new_fit_image(&fit_tmp, fit->rx, fit->ry, 1)) {
+		clearfits(dark_tmp);
+		return -1.0;
+	}
 
 	copyfits(dark, dark_tmp, CP_ALLOC | CP_EXTRACT, 0);
 	copyfits(fit, fit_tmp, CP_ALLOC | CP_EXTRACT, 0);
@@ -929,6 +783,8 @@ static double goldenSectionSearch(fits *brut, fits *dark, double a, double b,
 	do {
 		fc = evaluateNoiseOfCalibratedImage(brut, dark, c);
 		fd = evaluateNoiseOfCalibratedImage(brut, dark, d);
+		if (fc < 0.0 || fd < 0.0)
+			return -1.0;
 		if (fc < fd) {
 			b = d;
 			d = c;
@@ -964,13 +820,16 @@ static int darkOptimization(fits *brut, fits *dark, fits *offset) {
 	double k;
 	double lo = 0.0;
 	double up = 2.0;
+	fits *dark_tmp = NULL;
 
-	fits *dark_tmp = calloc(1, sizeof(fits));
-	new_fit_image(dark_tmp, dark->rx, dark->ry, 1);
+	if (new_fit_image(&dark_tmp, dark->rx, dark->ry, 1))
+		return -1;
 	copyfits(dark, dark_tmp, CP_ALLOC | CP_EXTRACT, 0);
 
 	/* Minimization of background noise to find better k */
 	k = goldenSectionSearch(brut, dark_tmp, lo, up, 1E-3);
+	if (k < 0.0)
+		return -1;
 
 	siril_log_message(_("Dark optimization: %.3lf\n"), k);
 	/* Multiply coefficient to master-dark */
@@ -1196,7 +1055,6 @@ void show_FITS_header(fits *fit) {
 		show_data_dialog(fit->header, "FITS Header");
 }
 
-#ifdef HAVE_OPENCV
 /* These functions do not more than resize_gaussian and rotate_image
  * except for console outputs. 
  * Indeed, siril_log_message seems not working in a cpp file */
@@ -1277,8 +1135,6 @@ int verbose_rotate_image(fits *image, double angle, int interpolation,
 	return 0;
 }
 
-#endif
-
 /* This function computes wavelets with the number of Nbr_Plan and
  * extracts plan "Plan" in fit parameters */
 
@@ -1479,7 +1335,8 @@ void apply_banding_to_sequence(struct banding_data *banding_args) {
 	args->save_hook = NULL;
 	args->image_hook = banding_image_hook;
 	args->idle_function = NULL;
-	args->description = "Banding Reduction";
+	args->stop_on_error = FALSE;
+	args->description = _("Banding Reduction");
 	args->has_output = TRUE;
 	args->new_seq_prefix = banding_args->seqEntry;
 	args->load_new_sequence = TRUE;
@@ -1530,26 +1387,15 @@ int BandingEngine(fits *fit, double sigma, double amount, gboolean protect_highl
 	int chan, row, i;
 	WORD *line, *fixline;
 	double minimum = DBL_MAX, globalsigma = 0.0;
-	fits *fiximage;
+	fits *fiximage = NULL;
 	double invsigma = 1.0 / sigma;
 
-	fiximage = calloc(1, sizeof(fits));
-	if (fiximage == NULL) {
-		fprintf(stderr, "BandingEngine: error allocating data\n");
-		return 1;
-	}
-
 	if (applyRotation) {
-#ifdef HAVE_OPENCV
 		cvRotateImage(fit, 90.0, -1, OPENCV_LINEAR);
-#else
-		siril_log_message(_("Rotation is only possible when Siril has been compiled with OpenCV support.\n"));
-		free(fiximage);
-		return 1;
-#endif
 	}
 
-	new_fit_image(fiximage, fit->rx, fit->ry, fit->naxes[2]);
+	if (new_fit_image(&fiximage, fit->rx, fit->ry, fit->naxes[2]))
+		return 1;
 
 	for (chan = 0; chan < fit->naxes[2]; chan++) {
 		imstats *stat = statistics(fit, chan, NULL, STATS_BASIC | STATS_MAD, STATS_ZERO_NULLCHECK);
@@ -1607,10 +1453,8 @@ int BandingEngine(fits *fit, double sigma, double amount, gboolean protect_highl
 	imoper(fit, fiximage, OPER_ADD);
 
 	clearfits(fiximage);
-#ifdef HAVE_OPENCV
 	if (applyRotation)
 		cvRotateImage(fit, -90.0, -1, OPENCV_LINEAR);
-#endif
 	return 0;
 }
 
@@ -1633,15 +1477,7 @@ int backgroundnoise(fits* fit, double sigma[]) {
 	}
 
 	copyfits(fit, waveimage, CP_ALLOC | CP_FORMAT | CP_COPYA, 0);
-#ifdef HAVE_OPENCV	// a bit faster
 	cvComputeFinestScale(waveimage);
-#else
-	if (get_wavelet_layers(waveimage, 4, 0, TO_PAVE_BSPLINE, -1)) {
-		siril_log_message(_("Siril cannot evaluate the noise in the image\n"));
-		clearfits(waveimage);
-		return 1;
-	}
-#endif
 
 	for (layer = 0; layer < fit->naxes[2]; layer++) {
 		imstats *stat = statistics(waveimage, layer, NULL, STATS_BASIC, STATS_ZERO_NULLCHECK);
@@ -1768,4 +1604,21 @@ gpointer noise(gpointer p) {
 	return GINT_TO_POINTER(0);
 }
 
+gpointer LRdeconv(gpointer p) {
+	struct RL_data *args = (struct RL_data *) p;
+	struct timeval t_start, t_end;
 
+	siril_log_color_message(_("Lucy-Richardson deconvolution: processing...\n"), "red");
+	gettimeofday(&t_start, NULL);
+
+	cvLucyRichardson(args->fit, args->sigma, args->iter);
+
+	gettimeofday(&t_end, NULL);
+	show_time(t_start, t_end);
+
+	gdk_threads_add_idle(end_generic, args);
+	adjust_cutoff_from_updated_gfit();
+	redraw(com.cvport, REMAP_ALL);
+	redraw_previews();
+	return 0;
+}
