@@ -4,6 +4,8 @@
 #  include <config.h>
 #endif
 
+#include <glib.h>
+#include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #include <fitsio.h>	// fitsfile
 #include <gsl/gsl_histogram.h>
@@ -12,11 +14,6 @@
 #endif
 #include <libintl.h>
 
-// PATH_MAX is not available on Hurd at least
-#ifndef PATH_MAX
-#define PATH_MAX 4096
-#endif
-
 #define _(String) gettext (String)
 #define gettext_noop(String) String
 #define N_(String) gettext_noop (String)
@@ -26,11 +23,13 @@
    ({ __typeof__ (a) _a = (a); \
        __typeof__ (b) _b = (b); \
      _a > _b ? _a : _b; })
+
 #undef min
  #define min(a,b) \
    ({ __typeof__ (a) _a = (a); \
        __typeof__ (b) _b = (b); \
      _a < _b ? _a : _b; })
+
 #define SQR(x) ((x)*(x))
 
 #define USHRT_MAX_DOUBLE ((double)USHRT_MAX)
@@ -78,7 +77,6 @@ typedef unsigned short WORD;		// default type for internal image data
 #define	STATS_ZERO_NONE 0
 #define	STATS_ZERO_NULLCHECK (!STATS_ZERO_NONE)
 
-
 /* when requesting an image redraw, it can be asked to remap its data before redrawing it.
  * REMAP_NONE	doesn't remaps the data,
  * REMAP_ONLY	remaps only the current viewport (color channel) and the mixed (RGB) image
@@ -94,7 +92,6 @@ enum {
 	N_COLUMNS_CONVERT
 };
 
-
 typedef enum {
 	TYPEUNDEF=(1 << 1),
 	TYPEFITS= (1 << 2),
@@ -114,8 +111,8 @@ typedef enum {
 #define USE_DARK	0x01
 #define USE_FLAT	0x02
 #define USE_OFFSET	0x04
-#define USE_COSME   0x08	/* cosmetic correction */
-#define USE_OPTD    0x10	/* dark optimization */
+#define USE_COSME	0x08	/* cosmetic correction */
+#define USE_OPTD	0x10	/* dark optimization */
 
 /* cookies for the file chooser */
 #define OD_NULL 	0
@@ -139,8 +136,8 @@ typedef enum {
 #define GREEN_VPORT 	1
 #define BLUE_VPORT 	2
 #define RGB_VPORT 	3
-#define MAXGRAYVPORT 	3			// 3 gray vports supported only (R, G, B)
-#define MAXCOLORVPORT	1			// 1 color vport supported only (RGB)
+#define MAXGRAYVPORT 	3	// 3 gray vports supported only (R, G, B)
+#define MAXCOLORVPORT	1	// 1 color vport supported only (RGB)
 #define MAXVPORT 	MAXGRAYVPORT + MAXCOLORVPORT
 
 /* defines for copyfits actions */
@@ -170,22 +167,21 @@ typedef enum {
 
 #define PREVIEW_NB 2
 
+/* special values for com.seq.current, the currently loaded image of the
+ * sequence. Negative values can be used to indicate that a specific image is
+ * loaded while there is a sequence or not in com.seq */
 #define RESULT_IMAGE -1		// used as current image index in a sequence
 				// when the result of a processing is displayed
 #define UNRELATED_IMAGE -2	// image loaded while a sequence was loaded too
+#define SCALED_IMAGE -3		// the single image has a different size than
+				// the loaded sequence
 
-#define MAX_STARS 50000
+#define MAX_STARS 50000		// maximum length of com.stars
 
 /* constants for loglut function */
 #define LOG 1
 #define EXP -1
 
-#define PROGRESS_NONE -2.0		// don't update the progress bar value
-#define PROGRESS_PULSATE -1.0		// pulsate the progress bar
-#define PROGRESS_RESET 0.0		// reset the progress bar
-#define PROGRESS_DONE 1.0		// fill the progress bar
-#define PROGRESS_TEXT_RESET ""		// reset the progress bar's text
-
 typedef struct imdata imgdata;
 typedef struct registration_data regdata;
 typedef struct layer_info_struct layer_info;
@@ -314,7 +310,7 @@ struct preprocessing_data {
 
 /* registration data, exists once for each image and each layer */
 struct registration_data {
-	int shiftx, shifty;	// we could have a subpixel precision, but is it needed? saved
+	float shiftx, shifty;	// we could have a subpixel precision, but is it needed? saved
 	float rot_centre_x, rot_centre_y;	// coordinates for the rotation centre, saved
 	float angle;		// angle for the rotation, saved
 	fitted_PSF *fwhm_data;	// used in PSF/FWHM registration, not saved
@@ -325,27 +321,25 @@ struct registration_data {
 	double quality;
 };
 
+/* see explanation about sequence and single image management in io/sequence.c */
+
 struct sequ {
 	char *seqname;		// name of the sequence, as in name.seq
 	int number;		// number of images in the sequence
 	int selnum;		// number of selected images in the sequence
 	int fixed;		// fixed length of image index in filename (like %3d)
-	int nb_layers;		// number of layers embedded in each image file
+	int nb_layers;		// number of layers embedded in each image file, -1 if unknown
 	unsigned int rx;	// first image width
 	unsigned int ry;	// first image height
-	layer_info *layers;	// info about layers
+	layer_info *layers;	// info about layers, may be null if nb_layers is unknown
 	int reference_image;	// reference image for registration
 	imgdata *imgparam;	// a structure for each image of the sequence
-	regdata **regparam;	// *regparam[nb_layers]
+	regdata **regparam;	// *regparam[nb_layers], may be null if nb_layers is unknown
 	/* beg and end are used prior to imgparam allocation, hence their usefulness */
 	int beg;		// imgparam[0]->filenum
 	int end;		// imgparam[number-1]->filenum
 	double exposure;	// exposure of frames (we assume they are all identical)
 
-	/* registration previsualisation and manual alignment data */
-	int previewX[PREVIEW_NB], previewY[PREVIEW_NB];	// center, -1 is uninitialized value
-	int previewW[PREVIEW_NB], previewH[PREVIEW_NB];	// 0 is uninitialized value
-
 	sequence_type type;
 	struct ser_struct *ser_file;
 #if defined(HAVE_FFMS2_1) || defined(HAVE_FFMS2_2)
@@ -362,8 +356,13 @@ struct sequ {
 	fits *dark;		// the image containing dark data
 	fits *flat;		// the image containing flat data
 	char *ppprefix;		// prefix for filename output of preprocessing
-	int current;		// file number currently loaded in wfit (or displayed)
-	//struct registration_method reg_method;	// is it the right place for that?
+	int current;		// file number currently loaded in gfit (or displayed)
+
+	/* registration previsualisation and manual alignment data */
+	int previewX[PREVIEW_NB], previewY[PREVIEW_NB];	// center, -1 is uninitialized value
+	int previewW[PREVIEW_NB], previewH[PREVIEW_NB];	// 0 is uninitialized value
+
+	double upscale_at_stacking;// up-scale factor during stacking (see #215)
 	
 	gboolean needs_saving;	// a dirty flag for the sequence, avoid saving it too often
 
@@ -540,8 +539,8 @@ struct cominf {
 	guchar *refimage_regbuffer;	// the graybuf[registration_layer] of the reference image
 	cairo_surface_t *refimage_surface;
 
-	char *wd;			// working directory, where images and sequences are
-	char *initfile;			// the path of the init file
+	gchar *wd;			// working directory, where images and sequences are
+	gchar *initfile;			// the path of the init file
 	
 	char *ext;		// FITS extension used in SIRIL
 	int len_ext;
@@ -570,7 +569,7 @@ struct cominf {
 	int hist_size;			// allocated size
 	int hist_current;		// current index
 	int hist_display;		// displayed index
-	char *swap_dir;
+	gchar *swap_dir;
 
 	libraw raw_set;			// the libraw settings
 	struct debayer_config debayer;	// debayer settings
@@ -581,6 +580,11 @@ struct cominf {
 
 	gsl_histogram *layers_hist[MAXVPORT]; // current image's histograms
 
+	gboolean stfComputed;	// Ugly Flag to know if stf parameters are already computed
+	double stfShadows;
+	double stfHighlights;
+	double stfM;
+
 	fitted_PSF **stars;		// list of stars detected in the current image
 	gboolean star_is_seqdata;	// the only star in stars belongs to seq, don't free it
 	int selected_star;		// current selected star in the GtkListStore
