@@ -1,7 +1,6 @@
 #ifndef CALLBACKS_H
 #define CALLBACKS_H
 
-#include <gtk/gtk.h>
 #include <sys/time.h>
 #include "core/siril.h"	// for sliders_mode
 
@@ -10,9 +9,6 @@ GtkWidget* lookup_widget (const gchar *widget_name);
 void initialize_shortcuts();
 void fill_about_dialog();
 void initialize_remap();
-char* siril_log_message(const char* format, ...);
-char* siril_log_color_message(const char* format, const char* color, ...);
-void show_time(struct timeval, struct timeval);
 void set_cutoff_sliders_max_values();		// was set_upper_minmax
 void set_cutoff_sliders_values();		// was set_ranges
 void set_sliders_value_to_gfit();
@@ -23,6 +19,7 @@ void adjust_refimage(int n);
 int adjust_sellabel();
 void set_GUI_CWD();
 void set_GUI_MEM(unsigned long size);
+void set_GUI_DiskSpace(double mem);
 void set_GUI_misc();
 void initialize_preprocessing();
 void update_MenuItem();
@@ -57,7 +54,6 @@ void show_rgb_window();
 void hide_rgb_window();
 void set_cursor_waiting(gboolean waiting);
 
-void set_progress_bar_data(const char *text, double percent);
 void zoomcombo_update_display_for_zoom();
 void initialize_FITS_name_entries();
 void adjust_vport_size_to_image();
@@ -67,6 +63,7 @@ void set_output_filename_to_sequence_name();
 void close_tab();
 void activate_tab(int vport);
 void control_window_switch_to_tab(main_tabs tab);
+void on_comboboxreglayer_changed(GtkComboBox *widget, gpointer user_data);
 
 void set_prepro_button_sensitiveness();
 
