@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2017 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2018 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -28,7 +28,6 @@
 #include <unistd.h>
 #include <sys/time.h>
 #include <sys/types.h>
-#include <dirent.h>
 #include "core/siril.h"
 #include "core/proto.h"
 #include "core/processing.h"
@@ -37,6 +36,7 @@
 #include "io/sequence.h"
 #include "io/ser.h"
 #include "gui/callbacks.h"
+#include "gui/progress_and_log.h"
 #include "algos/demosaicing.h"
 
 #define MAX_OF_EXTENSIONS 50	// actual size of supported_extensions
@@ -80,6 +80,12 @@ char *filter_pattern[] = {
 	"GRBG"
 };
 
+enum {
+	COLUMN_CONV_FILENAME,		// gchar[]
+	COLUMN_CONV_DATE,			// gchar[]
+	N_COLUMNS
+};
+
 static gpointer convert_thread_worker(gpointer p);
 static gboolean end_convert_idle(gpointer p);
 
@@ -120,6 +126,28 @@ void list_format_available() {
 #endif
 }
 
+static gint sort_conv_tree(GtkTreeModel *model, GtkTreeIter *a, GtkTreeIter *b,
+		gpointer user_data) {
+	gchar *name_a, *name_b;
+	gchar *collate_key1, *collate_key2;
+	gint ret;
+
+	gtk_tree_model_get(model, a, 0,	&name_a, -1);
+	gtk_tree_model_get(model, b, 0,	&name_b, -1);
+
+	collate_key1  = g_utf8_collate_key_for_filename(name_a, strlen(name_a));
+	collate_key2  = g_utf8_collate_key_for_filename(name_b, strlen(name_b));
+
+	ret = g_strcmp0(collate_key1, collate_key2);
+
+	g_free(collate_key1);
+	g_free(collate_key2);
+	g_free(name_a);
+	g_free(name_b);
+
+	return ret;
+}
+
 void check_for_conversion_form_completeness() {
 	static GtkTreeView *tree_convert = NULL;
 	GtkTreeIter iter;
@@ -133,6 +161,12 @@ void check_for_conversion_form_completeness() {
 	model = gtk_tree_view_get_model(tree_convert);
 	valid = gtk_tree_model_get_iter_first(model, &iter);
 	gtk_widget_set_sensitive (go_button, destroot && destroot[0] != '\0' && valid);
+
+	/* we override the sort function in order to provide natural sort order */
+	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(model),
+			COLUMN_CONV_FILENAME, (GtkTreeIterCompareFunc) sort_conv_tree, NULL,
+			NULL);
+
 	update_statusbar_convert();
 }
 
@@ -171,7 +205,8 @@ void on_convtoroot_changed (GtkEditable *editable, gpointer user_data){
 	if (!multiple_ser)
 		multiple_ser = lookup_widget("multipleSER");
 	if (destroot) g_free(destroot);
-	destroot = g_strdup(name);
+
+	destroot = g_str_to_ascii(name, NULL); // we want to avoid special char
 
 	const char *ext = get_filename_ext(destroot);
 	if (ext && !g_ascii_strcasecmp(ext, "ser")) {
@@ -423,7 +458,7 @@ int count_selected_files() {
 
 struct _convert_data {
 	struct timeval t_start;
-	DIR *dir;
+	GDir *dir;
 	GList *list;
 	int start;
 	int total;
@@ -432,7 +467,8 @@ struct _convert_data {
 };
 
 void on_convert_button_clicked(GtkButton *button, gpointer user_data) {
-	DIR *dir;
+	GDir *dir;
+	GError *error = NULL;
 	gchar *file_data, *file_date;
 	const gchar *indice;
 	static GtkTreeView *tree_convert = NULL;
@@ -486,9 +522,10 @@ void on_convert_button_clicked(GtkButton *button, gpointer user_data) {
 		set_cursor_waiting(FALSE);
 		return;
 	}
-	if((dir = opendir(com.wd)) == NULL){
+	if((dir = g_dir_open(com.wd, 0, &error)) == NULL){
 		tmpmsg = siril_log_message(_("Conversion: error opening working directory %s.\n"), com.wd);
 		show_dialog(tmpmsg, _("Error"), "gtk-dialog-error");
+		fprintf (stderr, "Conversion: %s\n", error->message);
 		set_cursor_waiting(FALSE);
 		return ;
 	}
@@ -546,6 +583,7 @@ static gpointer convert_thread_worker(gpointer p) {
 		else g_snprintf(msg_bar, 256, _("Converting %s..."), src_filename);
 
 		imagetype = get_type_for_extension(src_ext);
+		com.filter = (int) imagetype;
 		if (imagetype == TYPEUNDEF) {
 			char msg[512];
 			siril_log_message(_("FILETYPE IS NOT SUPPORTED, CANNOT CONVERT: %s\n"), src_ext);
@@ -625,22 +663,24 @@ static gpointer convert_thread_worker(gpointer p) {
 		}
 		else {	// single image
 			fits *fit = any_to_new_fits(imagetype, src_filename, args->compatibility);
-			if (convflags & CONVDSTSER) {
-				if (convflags & CONV1X1)
-					keep_first_channel_from_fits(fit);
-				if (ser_write_frame_from_fit(ser_file, fit, args->nb_converted)) {
-					siril_log_message(_("Error while converting to SER (no space left?)\n"));
-					break;
-				}
-			} else {
-				g_snprintf(dest_filename, 128, "%s%05d", destroot, indice++);
-				if (save_to_target_fits(fit, dest_filename)) {
-					siril_log_message(_("Error while converting to FITS (no space left?)\n"));
-					break;
+			if (fit) {
+				if (convflags & CONVDSTSER) {
+					if (convflags & CONV1X1)
+						keep_first_channel_from_fits(fit);
+					if (ser_write_frame_from_fit(ser_file, fit, args->nb_converted)) {
+						siril_log_message(_("Error while converting to SER (no space left?)\n"));
+						break;
+					}
+				} else {
+					g_snprintf(dest_filename, 128, "%s%05d", destroot, indice++);
+					if (save_to_target_fits(fit, dest_filename)) {
+						siril_log_message(_("Error while converting to FITS (no space left?)\n"));
+						break;
+					}
 				}
+				clearfits(fit);
+				free(fit);
 			}
-			clearfits(fit);
-			free(fit);
 		}
 
 		set_progress_bar_data(msg_bar, progress/((double)args->total));
@@ -680,6 +720,7 @@ static gboolean end_convert_idle(gpointer p) {
 	show_time(args->t_start, t_end);
 	stop_processing_thread();
 	g_list_free_full(args->list, g_free);
+	g_dir_close(args->dir);
 	free(args);
 	return FALSE;
 }
