@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2017 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2018 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -25,11 +25,15 @@
 #include <ctype.h>
 #include <string.h>
 #include <gsl/gsl_statistics.h>
+#ifdef _WIN32
+#include <windows.h>
+#endif
 
 #include "core/siril.h"
 #include "core/proto.h"
 #include "io/sequence.h"
 #include "gui/callbacks.h"
+#include "gui/progress_and_log.h"
 
 static char *MIPSHI[] = {"MIPS-HI", "CWHITE", NULL };
 static char *MIPSLO[] = {"MIPS-LO", "CBLACK", NULL };
@@ -172,6 +176,53 @@ static void read_fits_header(fits *fit) {
 	fits_read_key(fit->fptr, TUSHORT, "DFT_RY", &(fit->dft_ry), NULL, &status);
 }
 
+static gchar *fits_fname(const gchar *path) {
+	gchar *str;
+#ifdef _WIN32
+	wchar_t *wpath;
+
+	wpath = g_utf8_to_utf16(path, -1, NULL, NULL, NULL);
+	if (wpath == NULL)
+		return NULL;
+
+	// use the short DOS 8.3 path name to avoid problems converting UTF-16 filenames to the ANSI filenames expected by CFITTSIO
+	DWORD shortlen = GetShortPathNameW(wpath, 0, 0);
+
+	if (shortlen) {
+		LPWSTR shortpath = g_new(WCHAR, shortlen);
+		GetShortPathNameW(wpath, shortpath, shortlen);
+		int slen = WideCharToMultiByte(CP_OEMCP, WC_NO_BEST_FIT_CHARS,
+				shortpath, shortlen, 0, 0, 0, 0);
+		str = g_new(gchar, slen + 1);
+		WideCharToMultiByte(CP_OEMCP, WC_NO_BEST_FIT_CHARS, shortpath, shortlen,
+				str, slen, 0, 0);
+		g_free(shortpath);
+	} else {
+		str = NULL;
+	}
+	g_free(wpath);
+#else // _WIN32
+	str = g_strdup(path);
+#endif // _WIN32
+	return str;
+}
+
+static void report_fits_error(int status) {
+	if (status) {
+		char errmsg[FLEN_ERRMSG];
+		while (fits_read_errmsg(errmsg)) {
+			siril_log_message(_("FITS error: %s\n"), errmsg);
+		}
+	}
+}
+
+static int siril_fits_open_diskfile(fitsfile **fptr, const char *filename, int iomode, int *status) {
+	gchar *fname = fits_fname(filename);
+	int ret = fits_open_diskfile(fptr, fname, iomode, status);
+	g_free(fname);
+	return ret;
+}
+
 // return 0 on success, fills realname if not NULL with the opened file's name
 int readfits(const char *filename, fits *fit, char *realname) {
 	int status;
@@ -209,7 +260,8 @@ int readfits(const char *filename, fits *fit, char *realname) {
 	if (realname)
 		strcpy(realname, name);
 	status = 0;
-	fits_open_diskfile(&(fit->fptr), name, READONLY, &status);
+
+	siril_fits_open_diskfile(&(fit->fptr), name, READONLY, &status);
 	if (status) {
 		report_fits_error(status);
 		free(name);
@@ -448,15 +500,6 @@ void clearfits(fits *fit) {
 	memset(fit, 0, sizeof(fits));
 }
 
-void report_fits_error(int status) {
-	if (status) {
-		char errmsg[FLEN_ERRMSG];
-		while (fits_read_errmsg(errmsg)) {
-			siril_log_message(_("FITS error: %s\n"), errmsg);
-		}
-	}
-}
-
 /* Read a rectangular section of a FITS image in Siril's format, pointed by its
  * exact filename. Only layer layer is read. */
 int readfits_partial(const char *filename, int layer, fits *fit,
@@ -468,7 +511,7 @@ int readfits_partial(const char *filename, int layer, fits *fit,
 	int dataType = TUSHORT;
 
 	status = 0;
-	if (fits_open_diskfile(&(fit->fptr), filename, READONLY, &status))
+	if (siril_fits_open_diskfile(&(fit->fptr), filename, READONLY, &status))
 		report_fits_error(status);
 	if (status)
 		return status;
@@ -636,7 +679,7 @@ int read_opened_fits_partial(sequence *seq, int layer, int index, WORD *buffer,
 
 int fits_get_date_obs(const char *name, fits *f) {
 	int status = 0;
-	if (fits_open_diskfile(&(f->fptr), name, READONLY, &status))
+	if (siril_fits_open_diskfile(&(f->fptr), name, READONLY, &status))
 		report_fits_error(status);
 	if (status)
 		return status;
@@ -669,7 +712,7 @@ int savefits(const char *name, fits *f) {
 		snprintf(filename, 255, "%s", name);
 	}
 
-	unlink(filename); /* Delete old file if it already exists */
+	g_unlink(filename); /* Delete old file if it already exists */
 
 	status = 0;
 	if (fits_create_diskfile(&(f->fptr), filename, &status)) { /* create new FITS file */
@@ -1191,47 +1234,59 @@ void extract_region_from_fits(fits *from, int layer, fits *to,
 	to->bitpix = (from->bitpix ? from->bitpix : USHRT_MAX);
 }
 
-int new_fit_image(fits *fit, int width, int height, int nblayer) {
+/* creates a new fit image from scratch (NULL fit) or into a fits * previously
+ * allocated, with non-cleared (random) data. */
+int new_fit_image(fits **fit, int width, int height, int nblayer) {
 	gint npixels;
 	WORD *data;
 	assert(width > 0);
 	assert(height > 0);
-	assert(nblayer <= 3);
+	assert(nblayer == 1 || nblayer == 3);
 
 	npixels = width * height;
-	data = calloc(npixels, sizeof(WORD) * nblayer);
+	data = malloc(npixels * nblayer * sizeof(WORD));
 
-	if (data != NULL) {
-		clearfits(fit);
-		fit->bitpix = USHORT_IMG;
-		if (nblayer > 1)
-			fit->naxis = nblayer;
-		else
-			fit->naxis = 2;
-		fit->rx = width;
-		fit->ry = height;
-		fit->naxes[0] = width;
-		fit->naxes[1] = height;
-		fit->naxes[2] = nblayer;
-		fit->data = data;
-		fit->pdata[RLAYER] = fit->data;
-		if (nblayer > 1) {
-			fit->pdata[GLAYER] = fit->data + npixels;
-			fit->pdata[BLAYER] = fit->data + npixels * 2;
+	if (data == NULL) {
+		fprintf(stderr, "Could not allocate memory\n");
+		return -1;
+	}
+
+	if (*fit)
+		clearfits(*fit);
+	else {
+		*fit = calloc(1, sizeof(fits));
+		if (!*fit) {
+			free(data);
+			return -1;
 		}
-		else {
-			fit->pdata[GLAYER] = fit->data;
-			fit->pdata[BLAYER] = fit->data;
-		}
-		return 0;
-	} else
-		return 1;
+	}
+
+	(*fit)->bitpix = USHORT_IMG;
+	if (nblayer > 1)
+		(*fit)->naxis = 3;
+	else (*fit)->naxis = 2;
+	(*fit)->rx = width;
+	(*fit)->ry = height;
+	(*fit)->naxes[0] = width;
+	(*fit)->naxes[1] = height;
+	(*fit)->naxes[2] = nblayer;
+	(*fit)->data = data;
+	(*fit)->pdata[RLAYER] = (*fit)->data;
+	if (nblayer > 1) {
+		(*fit)->pdata[GLAYER] = (*fit)->data + npixels;
+		(*fit)->pdata[BLAYER] = (*fit)->data + npixels * 2;
+	}
+	else {
+		(*fit)->pdata[GLAYER] = (*fit)->data;
+		(*fit)->pdata[BLAYER] = (*fit)->data;
+	}
+	return 0;
 }
 
 void keep_first_channel_from_fits(fits *fit) {
 	if (fit->naxis == 1)
 		return;
-	fit->naxis = 1;
+	fit->naxis = 2;
 	fit->naxes[2] = 1;
 	fit->data = realloc(fit->data, fit->rx * fit->ry * sizeof(WORD));
 	fit->pdata[RLAYER] = fit->data;
