@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2017 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2018 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -30,20 +30,21 @@
 #include <stdio.h>
 #include <string.h>
 #include <assert.h>
-#ifdef WIN32
+#ifdef _WIN32
 #include <io.h>
 #endif
 
 #include "core/siril.h"
 #include "core/proto.h"
 #include "gui/callbacks.h"
+#include "gui/progress_and_log.h"
 #include "algos/demosaicing.h"
 #include "io/ser.h"
 
 #ifndef O_BINARY
 #define O_BINARY 0
 #endif
-#ifdef WIN32
+#ifdef _WIN32
 #define lseek64 _lseeki64
 #else
 #define lseek64 lseek
@@ -69,7 +70,7 @@ static char *ser_timestamp(uint64_t timestamp) {
 
 	t = gmtime(&secs);
 
-	sprintf(str, "%04d-%02d-%02d %02d:%02d:%02d.%03d", t->tm_year + 1900,
+	sprintf(str, "%04d-%02d-%02dT%02d:%02d:%02d.%03d", t->tm_year + 1900,
 			t->tm_mon + 1, t->tm_mday, t->tm_hour, t->tm_min, t->tm_sec, ms);
 
 	return str;
@@ -81,7 +82,7 @@ static int display_date(uint64_t timestamp, char *txt) {
 		return -1;
 
 	char *str = ser_timestamp(timestamp);
-	printf("%s%s\n", txt, str);
+	fprintf(stdout, "%s%s\n", txt, str);
 	free(str);
 	return 0;
 }
@@ -90,7 +91,7 @@ static int display_date(uint64_t timestamp, char *txt) {
  * This is not thread-safe
  */
 static time_t __timegm(struct tm *tm) {
-#ifdef WIN32
+#ifdef _WIN32
 	return _mkgmtime(tm);
 #else
 	time_t ret;
@@ -116,20 +117,22 @@ static int FITS_date_key_to_Unix_time(char *date, uint64_t *utc,
 		uint64_t *local) {
 	struct tm timeinfo = { };
 	time_t ut, t;
-	int year = 0, month = 0, day = 0, hour = 0, min = 0, sec = 0, ms = 0;
+	int year = 0, month = 0, day = 0, hour = 0, min = 0, ms = 0;
+	float sec = 0.0;
 
 	if (date[0] == '\0')
 		return -1;
 
-	sscanf(date, "%04d-%02d-%02dT%02d:%02d:%02d.%02d", &year, &month, &day, &hour,
-			&min, &sec, &ms);
+	sscanf(date, "%04d-%02d-%02dT%02d:%02d:%f", &year, &month, &day, &hour,
+			&min, &sec);
 
 	timeinfo.tm_year = year - 1900;
 	timeinfo.tm_mon = month - 1;
 	timeinfo.tm_mday = day;
 	timeinfo.tm_hour = hour;
 	timeinfo.tm_min = min;
-	timeinfo.tm_sec = sec;
+	timeinfo.tm_sec = (int) sec;
+	ms = ((int) (sec * 1000) % 1000);
 
 	// Hopefully these are not needed
 	timeinfo.tm_wday = 0;
@@ -140,14 +143,14 @@ static int FITS_date_key_to_Unix_time(char *date, uint64_t *utc,
 	ut = __timegm(&timeinfo);
 	ut *= ticksPerSecond;
 	ut += epochTicks;
-	ut += ms * 100000;
+	ut += ms * 10000;
 	*utc = (uint64_t) ut;
 
 	/* get local time from timeinfo* */
 	t = mktime(&timeinfo);
 	t *= ticksPerSecond;
 	t += epochTicks;
-	t += ms * 100000;
+	t += ms * 10000;
 	*local = (uint64_t) t;
 
 	return 0;
@@ -182,36 +185,36 @@ static char *convert_color_id_to_char(ser_color color_id) {
 	}
 }
 
+/* reads timestamps from the trailer of the file and stores them in ser_file->ts */
 static int ser_read_timestamp(struct ser_struct *ser_file) {
-	int frame_size, i;
+	int i;
 	gboolean timestamps_in_order = TRUE;
 	uint64_t previous_ts = 0L;
-	int64_t filesize;
+	int64_t frame_size;
 
-	filesize = ser_file->filesize;
-	frame_size = ser_file->image_width * ser_file->image_height * ser_file->number_of_planes;
-	int64_t offset = SER_HEADER_LEN + (int64_t) frame_size * (int64_t) ser_file->byte_pixel_depth
-		* (int64_t) ser_file->frame_count;
+	ser_file->fps = -1.0;	// will be calculated from the timestamps
 
+	if (!ser_file->frame_count || ser_file->image_width <= 0 ||
+			ser_file->image_height <= 0 || ser_file->byte_pixel_depth <= 0 ||
+			!ser_file->number_of_planes)
+		return 0;
+
+	frame_size = ser_file->image_width *
+		ser_file->image_height * ser_file->number_of_planes;
+	int64_t offset = SER_HEADER_LEN + frame_size *
+		(int64_t)ser_file->byte_pixel_depth * (int64_t)ser_file->frame_count;
 	/* Check if file is large enough to have timestamps */
-	if (filesize >= offset + (8 * ser_file->frame_count)) {
+	if (ser_file->filesize >= offset + (8 * ser_file->frame_count)) {
 		ser_file->ts = calloc(8, ser_file->frame_count);
+		ser_file->ts_alloc = ser_file->frame_count;
 
 		// Seek to start of timestamps
 		for (i = 0; i < ser_file->frame_count; i++) {
-			if ((int64_t) -1 == lseek64(ser_file->fd, offset + (i * 8), SEEK_SET)) {
+			if ((int64_t)-1 == lseek64(ser_file->fd, offset+(i*8), SEEK_SET))
 				return -1;
-			}
-			char timestamp[8];
 
-			memset(timestamp, 0, sizeof(timestamp));
-			if (8 == read(ser_file->fd, timestamp, 8)) {
-				memcpy(&ser_file->ts[i], timestamp, 8);
-			} else {
-				// No valid frames per second value can be calculated
-				ser_file->fps = -1.0;
+			if (8 != read(ser_file->fd, &ser_file->ts[i], 8))
 				return 0;
-			}
 		}
 
 		/* Check order of Timestamps */
@@ -237,40 +240,34 @@ static int ser_read_timestamp(struct ser_struct *ser_file) {
 		}
 
 		if (timestamps_in_order) {
-			if (min_ts == max_ts) {
-				printf("Timestamps are all identical\n");
-			} else {
-				printf("Timestamps are all in order\n");
-			}
+			if (min_ts == max_ts)
+				fprintf(stdout, _("Warning: timestamps in the SER sequence are all identical.\n"));
+			else fprintf(stdout, _("Timestamps in the SER sequence are correctly ordered.\n"));
 		} else {
-			printf("Timestamps are not in order\n");
+			fprintf(stdout, _("Warning: timestamps in the SER sequence are not in the correct order.\n"));
 		}
 
 		ser_file->ts_min = min_ts;
 		ser_file->ts_max = max_ts;
-		uint64_t diff_ts = (ser_file->ts_max - ser_file->ts_min) / 1000.0; // Now in units of 100 us
-		if (diff_ts > 0) {
-			// There is a positive time difference between first and last timestamps
-			// We can calculate a frames per second value
-			ser_file->fps = ((double) (ser_file->frame_count - 1) * 10000)
-				/ (double) diff_ts;
-		} else {
-			// No valid frames per second value can be calculated
-			ser_file->fps = -1.0;
+		double diff_ts = (ser_file->ts_max - ser_file->ts_min) / 1000.0;
+		// diff_ts now in units of 100 us or ten thousandths of a second
+		if (diff_ts > 0.0) {
+			// There is a positive time difference between first and last
+			// timestamps, we can calculate a frames per second value
+			ser_file->fps = (ser_file->frame_count - 1) * 10000.0 / diff_ts;
 		}
 	} else {
-		printf("No timestamps stored in the file.\n");
-		ser_file->fps = -1.0;
+		fprintf(stdout, _("Warning: no timestamps stored in the SER sequence.\n"));
 	}
 	return 0;
 }
 
-static int ser_fix_broken_file(struct ser_struct *ser_file) {
+static int ser_recompute_frame_count(struct ser_struct *ser_file) {
 	int frame_count_calculated;
 	int64_t filesize = ser_file->filesize;
 
 	siril_log_message(_("Trying to fix broken SER file...\n"));
-	int frame_size = ser_file->image_width * ser_file->image_height;
+	int64_t frame_size = ser_file->image_width * ser_file->image_height;
 	if (frame_size == 0)
 		return 0;
 
@@ -303,7 +300,7 @@ static int ser_read_header(struct ser_struct *ser_file) {
 	lseek64(ser_file->fd, 0, SEEK_SET);
 
 	/* Read header (size of 178) */
-	if (SER_HEADER_LEN != read(ser_file->fd, header, sizeof(header))) {
+	if (SER_HEADER_LEN != read(ser_file->fd, header, sizeof header)) {
 		perror("read");
 		return -1;
 	}
@@ -331,12 +328,12 @@ static int ser_read_header(struct ser_struct *ser_file) {
 	else
 		ser_file->number_of_planes = 1;
 
-/* In some cases, oacapture, firecapture, ... crash before writing frame_count
- * data. Here we try to get the calculated frame count which has not been written
- * in the header. Then we fix the SER file
- */
+	/* In some cases, oacapture, firecapture, ... crash before writing
+	 * frame_count data. Here we try to get the calculated frame count
+	 * which has not been written in the header. Then we fix the SER file
+	 */
 	if (ser_file->frame_count == 0) {
-		ser_file->frame_count = ser_fix_broken_file(ser_file);
+		ser_file->frame_count = ser_recompute_frame_count(ser_file);
 
 		if (ser_file->frame_count > 0) {
 			if (ser_write_header(ser_file) == 0)
@@ -349,36 +346,38 @@ static int ser_read_header(struct ser_struct *ser_file) {
 	return 0;
 }
 
-static int ser_write_timestamp(struct ser_struct *ser_file) {
-	int frame_size, i;
+static int ser_write_timestamps(struct ser_struct *ser_file) {
+	int i;
+	int64_t frame_size;
 
-	if (ser_file->frame_count > 0) {
-		if (ser_file->ts) {
-			// Seek to start of timestamps
-			frame_size = ser_file->image_width * ser_file->image_height
-					* ser_file->number_of_planes;
-			int64_t offset = SER_HEADER_LEN
-					+ (int64_t) frame_size * (int64_t) ser_file->byte_pixel_depth
-							* (int64_t) ser_file->frame_count;
-			for (i = 0; i < ser_file->frame_count; i++) {
-				if ((int64_t) -1
-						== lseek64(ser_file->fd, offset + (i * 8), SEEK_SET)) {
-					return -1;
-				}
-				char timestamp[8];
+	if (!ser_file->frame_count || ser_file->image_width <= 0 ||
+			ser_file->image_height <= 0 || ser_file->byte_pixel_depth <= 0 ||
+			!ser_file->number_of_planes)
+		return -1;
 
-				memset(timestamp, 0, sizeof(timestamp));
-				memcpy(timestamp, &ser_file->ts[i], 8);
-				if (8 != write(ser_file->fd, timestamp, 8)) {
-					perror("WriteTimetamps:");
-					return -1;
-				}
+	if (ser_file->ts) {
+		// Seek to start of timestamps
+		frame_size = ser_file->image_width * ser_file->image_height
+			* ser_file->number_of_planes;
+		int64_t offset = SER_HEADER_LEN + frame_size * 
+			(int64_t)ser_file->byte_pixel_depth * (int64_t)ser_file->frame_count;
+
+		for (i = 0; i < ser_file->frame_count; i++) {
+			if (i >= ser_file->ts_alloc)
+				break;
+			if ((int64_t)-1 == lseek64(ser_file->fd, offset+(i*8), SEEK_SET)) {
+				return -1;
+			}
+			if (8 != write(ser_file->fd, &ser_file->ts[i], 8)) {
+				perror("write timestamps:");
+				return -1;
 			}
 		}
 	}
 	return 0;
 }
 
+/* (over)write the header of the opened file on the disk */
 static int ser_write_header(struct ser_struct *ser_file) {
 	char header[SER_HEADER_LEN];
 
@@ -403,9 +402,6 @@ static int ser_write_header(struct ser_struct *ser_file) {
 		perror("write");
 		return 1;
 	}
-
-	ser_write_timestamp(ser_file);
-
 	return 0;
 }
 
@@ -414,13 +410,11 @@ static void ser_write_header_from_fit(struct ser_struct *ser_file, fits *fit) {
 	ser_file->image_width = fit->rx;
 	ser_file->image_height = fit->ry;
 	fprintf(stdout, "setting SER image size as %dx%d\n", fit->rx, fit->ry);
-	if (fit->naxes[2] == 1) {
-		// maybe it's read as CFA...
-		ser_file->color_id = SER_MONO;
-	} else if (fit->naxes[2] == 3) {
+	// already managed during creation for monochrome formats
+	if (fit->naxes[2] == 3) {
 		ser_file->color_id = SER_RGB;
 	}
-	if (ser_file->color_id == SER_RGB || ser_file->color_id == SER_BGR)
+	if (ser_file->color_id == SER_RGB)
 		ser_file->number_of_planes = 3;
 	else ser_file->number_of_planes = 1;
 
@@ -445,26 +439,21 @@ static void ser_write_header_from_fit(struct ser_struct *ser_file, fits *fit) {
 		memset(ser_file->telescope, 0, 40);
 		memcpy(ser_file->telescope, fit->telescop, 40);
 	}
-	int ret = FITS_date_key_to_Unix_time(fit->date_obs, &ser_file->date_utc, &ser_file->date);
-	if (ret == -1)
+
+	if (FITS_date_key_to_Unix_time(fit->date_obs, &ser_file->date_utc, &ser_file->date) == -1)
 		FITS_date_key_to_Unix_time(fit->date, &ser_file->date_utc, &ser_file->date);
 }
 
-static int retrieveSERBayerPattern(ser_color pattern) {
-
+static int get_SER_Bayer_Pattern(ser_color pattern) {
 	switch (pattern) {
 	case SER_BAYER_RGGB:
 		return BAYER_FILTER_RGGB;
-		break;
 	case SER_BAYER_BGGR:
 		return BAYER_FILTER_BGGR;
-		break;
 	case SER_BAYER_GBRG:
 		return BAYER_FILTER_GBRG;
-		break;
 	case SER_BAYER_GRBG:
 		return BAYER_FILTER_GRBG;
-		break;
 	default:
 		return BAYER_FILTER_NONE;
 	}
@@ -473,7 +462,8 @@ static int retrieveSERBayerPattern(ser_color pattern) {
 /* once a buffer (data) has been acquired from the file, with frame_size pixels
  * read in it, depending on ser_file's endianess and pixel depth, data is
  * reorganized to match Siril's data format . */
-void ser_manage_endianess_and_depth(struct ser_struct *ser_file, WORD *data, int frame_size) {
+static void ser_manage_endianess_and_depth(struct ser_struct *ser_file,
+		WORD *data, int64_t frame_size) {
 	WORD pixel;
 	int i;
 	if (ser_file->byte_pixel_depth == SER_PIXEL_DEPTH_8) {
@@ -490,17 +480,40 @@ void ser_manage_endianess_and_depth(struct ser_struct *ser_file, WORD *data, int
 	}
 }
 
+static int ser_alloc_ts(struct ser_struct *ser_file, int frame_no) {
+	int retval = 0;
+#ifdef _OPENMP
+	omp_set_lock(&ser_file->ts_lock);
+#endif
+	if (ser_file->ts_alloc <= frame_no) {
+		uint64_t *new = realloc(ser_file->ts, (frame_no + 1) * 2 * sizeof(uint64_t));
+		if (!new) {
+			retval = 1;
+		} else {
+			ser_file->ts = new;
+			ser_file->ts_alloc = (frame_no + 1) * 2;
+		}
+	}
+#ifdef _OPENMP
+	omp_unset_lock(&ser_file->ts_lock);
+#endif
+	return retval;
+}
+
 /*
  * Public functions
  */
 
+/* set the timestamps of the ser_file using a list of timestamps in string form */
 void ser_convertTimeStamp(struct ser_struct *ser_file, GSList *timestamp) {
 	int i = 0;
+	if (ser_file->ts)
+		free(ser_file->ts);
 	ser_file->ts = calloc(8, ser_file->frame_count);
+	ser_file->ts_alloc = ser_file->frame_count;
 
-	while (timestamp) {
+	while (timestamp && i < ser_file->frame_count) {
 		uint64_t utc, local;
-
 		FITS_date_key_to_Unix_time(timestamp->data, &utc, &local);
 		timestamp = timestamp->next;
 		memcpy(&ser_file->ts[i], &utc, 8);
@@ -529,23 +542,40 @@ void ser_display_info(struct ser_struct *ser_file) {
 }
 
 int ser_write_and_close(struct ser_struct *ser_file) {
+	if (!ser_file->frame_count) {
+		siril_log_color_message(_("The SER sequence is being created with no image in it.\n"), "red");
+		char *filename = ser_file->filename;
+		ser_file->filename = NULL;
+		ser_close_file(ser_file);// closes, frees and zeroes
+		g_unlink(filename);
+		return -1;
+	}
 	ser_write_header(ser_file);	// writes the header
+	ser_write_timestamps(ser_file);	// writes the trailer
 	return ser_close_file(ser_file);// closes, frees and zeroes
 }
 
-/* ser_file must be allocated */
+/* ser_file must be allocated
+ * the file is created with no image size, the first image added will set it. */
 int ser_create_file(const char *filename, struct ser_struct *ser_file,
 		gboolean overwrite, struct ser_struct *copy_from) {
 	if (overwrite)
-		unlink(filename);
-	if ((ser_file->fd = open(filename, O_CREAT | O_RDWR | O_BINARY,
+		g_unlink(filename);
+	if ((ser_file->fd = g_open(filename, O_CREAT | O_RDWR | O_BINARY,
 			S_IWRITE | S_IREAD)) == -1) {
 		perror("open SER file for creation");
 		return 1;
 	}
 
+	ser_file->filename = strdup(filename);
+	ser_file->ts = NULL;
+	ser_file->ts_alloc = 0;
+	ser_file->fps = -1.0;
+	ser_file->frame_count = 0;	// incremented on image add
+
 	if (copy_from) {
-		memcpy(&ser_file->lu_id, &copy_from->lu_id, 28);
+		memcpy(&ser_file->lu_id, &copy_from->lu_id, 12);
+		memset(&ser_file->image_width, 0, 16);
 		memcpy(&ser_file->date, &copy_from->date, 8);
 		memcpy(&ser_file->date_utc, &copy_from->date_utc, 8);
 		ser_file->file_id = strdup(copy_from->file_id);
@@ -553,16 +583,11 @@ int ser_create_file(const char *filename, struct ser_struct *ser_file,
 		memcpy(ser_file->instrument, copy_from->instrument, 40);
 		memcpy(ser_file->telescope, copy_from->telescope, 40);
 		ser_file->byte_pixel_depth = copy_from->byte_pixel_depth;
-		ser_file->number_of_planes = copy_from->number_of_planes;
+		ser_file->number_of_planes = 0;	// used as an indicator of new SER
 
-		int i;
-		if (copy_from->ts) {
+		if (copy_from->ts && copy_from->frame_count > 0) {
 			ser_file->ts = calloc(8, copy_from->frame_count);
-			for (i = 0; i < copy_from->frame_count; i++)
-				ser_file->ts[i] = copy_from->ts[i];
-		} else {
-			ser_file->ts = NULL;
-			ser_file->fps = -1.0;
+			ser_file->ts_alloc = copy_from->frame_count;
 		}
 		/* we write the header now, but it should be written again
 		 * before closing in case the number of the image in the new
@@ -571,37 +596,29 @@ int ser_create_file(const char *filename, struct ser_struct *ser_file,
 	} else {	// new SER
 		ser_file->file_id = strdup("Made by Siril");
 		ser_file->lu_id = 0;
+		ser_file->color_id = SER_MONO;	// this is 0
 		ser_file->little_endian = SER_LITTLE_ENDIAN; // what will it do on big endian machine?
 		memset(ser_file->observer, 0, 40);
 		memset(ser_file->instrument, 0, 40);
 		memset(ser_file->telescope, 0, 40);
-		ser_file->ts = NULL;
 		memset(&ser_file->date, 0, 8);
 		memset(&ser_file->date_utc, 0, 8);
 		ser_file->number_of_planes = 0;	// used as an indicator of new SER
-
-		/* next operation should be ser_write_frame_from_fit, which writes with no
-		 * seek and expects to be after the header */
-		if ((int64_t) -1 == lseek64(ser_file->fd, SER_HEADER_LEN, SEEK_SET)) {
-			perror("seek");
-			return -1;
-		}
 	}
-	ser_file->filename = strdup(filename);
-	ser_file->frame_count = 0;	// incremented on image add
 #ifdef _OPENMP
 	omp_init_lock(&ser_file->fd_lock);
+	omp_init_lock(&ser_file->ts_lock);
 #endif
 	siril_log_message(_("Created SER file %s\n"), filename);
 	return 0;
 }
 
-int ser_open_file(char *filename, struct ser_struct *ser_file) {
+int ser_open_file(const char *filename, struct ser_struct *ser_file) {
 	if (ser_file->fd > 0) {
 		fprintf(stderr, "SER: file already opened, or badly closed\n");
 		return -1;
 	}
-	ser_file->fd = open(filename, O_RDWR | O_BINARY); // now we can fix broken file, so not O_RDONLY anymore
+	ser_file->fd = g_open(filename, O_RDWR | O_BINARY, 0); // now we can fix broken file, so not O_RDONLY anymore
 	if (ser_file->fd == -1) {
 		perror("SER file open");
 		return -1;
@@ -616,6 +633,7 @@ int ser_open_file(char *filename, struct ser_struct *ser_file) {
 
 #ifdef _OPENMP
 	omp_init_lock(&ser_file->fd_lock);
+	omp_init_lock(&ser_file->ts_lock);
 #endif
 	return 0;
 }
@@ -623,7 +641,7 @@ int ser_open_file(char *filename, struct ser_struct *ser_file) {
 int ser_close_file(struct ser_struct *ser_file) {
 	int retval = 0;
 	if (!ser_file)
-		return retval;
+		return -1;
 	if (ser_file->fd > 0) {
 		retval = close(ser_file->fd);
 		ser_file->fd = -1;
@@ -636,6 +654,7 @@ int ser_close_file(struct ser_struct *ser_file) {
 		free(ser_file->filename);
 #ifdef _OPENMP
 	omp_destroy_lock(&ser_file->fd_lock);
+	omp_destroy_lock(&ser_file->ts_lock);
 #endif
 	ser_init_struct(ser_file);
 	return retval;
@@ -645,43 +664,48 @@ void ser_init_struct(struct ser_struct *ser_file) {
 	memset(ser_file, 0, sizeof(struct ser_struct));
 }
 
-/* frame number starts at 0 */
+/* reads a frame on an already opened SER sequence.
+ * frame number starts at 0 */
 int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit) {
-	int retval, frame_size, i, j, swap = 0;
-	int64_t offset;
+	int retval = 0, i, j, swap = 0;
+	int64_t offset, frame_size;
+	size_t read_size;
 	WORD *olddata, *tmp;
 	if (!ser_file || ser_file->fd <= 0 || !ser_file->number_of_planes ||
 			!fit || frame_no < 0 || frame_no >= ser_file->frame_count)
 		return -1;
-	frame_size = ser_file->image_width * ser_file->image_height
-			* ser_file->number_of_planes;
+
+	frame_size = ser_file->image_width * ser_file->image_height *
+			ser_file->number_of_planes;
+	read_size = frame_size * ser_file->byte_pixel_depth;
+
 	olddata = fit->data;
 	if ((fit->data = realloc(fit->data, frame_size * sizeof(WORD))) == NULL) {
-		fprintf(stderr, "ser_read: error realloc %s %d\n", ser_file->filename,
+		fprintf(stderr, "ser_read: error realloc %s %ld\n", ser_file->filename,
 				frame_size);
 		if (olddata)
 			free(olddata);
 		return -1;
 	}
 
-	offset = SER_HEADER_LEN + (int64_t)frame_size *
+	offset = SER_HEADER_LEN	+ frame_size *
 		(int64_t)ser_file->byte_pixel_depth * (int64_t)frame_no;
 	/*fprintf(stdout, "offset is %lu (frame %d, %d pixels, %d-byte)\n", offset,
 	 frame_no, frame_size, ser_file->pixel_bytedepth);*/
 #ifdef _OPENMP
 	omp_set_lock(&ser_file->fd_lock);
 #endif
-	if ((int64_t) -1 == lseek64(ser_file->fd, offset, SEEK_SET)) {
-#ifdef _OPENMP
-		omp_unset_lock(&ser_file->fd_lock);
-#endif
-		return -1;
+	if ((int64_t)-1 == lseek64(ser_file->fd, offset, SEEK_SET)) {
+		perror("lseek in SER");
+		retval = -1;
+	} else {
+		if (read(ser_file->fd, fit->data, read_size) != read_size)
+			retval = -1;
 	}
-	retval = read(ser_file->fd, fit->data, frame_size * ser_file->byte_pixel_depth);
 #ifdef _OPENMP
 	omp_unset_lock(&ser_file->fd_lock);
 #endif
-	if (retval != frame_size * ser_file->byte_pixel_depth)
+	if (retval)
 		return -1;
 
 	ser_manage_endianess_and_depth(ser_file, fit->data, frame_size);
@@ -717,7 +741,7 @@ int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit) {
 		sensortmp = com.debayer.bayer_pattern;
 		if (com.debayer.use_bayer_header) {
 			sensor_pattern bayer;
-			bayer = retrieveSERBayerPattern(type_ser);
+			bayer = get_SER_Bayer_Pattern(type_ser);
 			if (bayer != com.debayer.bayer_pattern) {
 				if (bayer == BAYER_FILTER_NONE  && warning == FALSE) {
 					siril_log_color_message(_("No Bayer pattern found in the header file.\n"), "red");
@@ -764,56 +788,147 @@ int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit) {
 		siril_log_message(_("This type of Bayer pattern is not handled yet.\n"));
 		return -1;
 	}
+
+	/* copy the SER timestamp to the fits */
+	if (ser_file->ts) {
+		char *timestamp = ser_timestamp(ser_file->ts[frame_no]);
+		g_snprintf(fit->date_obs, FLEN_VALUE, "%s", timestamp);
+		free(timestamp);
+	}
+
 	fits_flip_top_to_bottom(fit);
 	return 0;
 }
 
+/* multi-type cropping, works in constant space if needed */
+#define crop_area_from_lines(BUFFER_TYPE) { \
+	int x, y, src, dst = 0; \
+	BUFFER_TYPE *inbuf = (BUFFER_TYPE *)read_buffer; \
+	BUFFER_TYPE *out = (BUFFER_TYPE *)outbuf; \
+	for (y = 0; y < area->h; y++) { \
+		src = y * ser_file->image_width + area->x; \
+		for (x = 0; x < area->w; x++) \
+			out[dst++] = inbuf[src++]; \
+	} \
+}
+
+/* multi-type RGB reordering, works in constant space if needed */
+#define crop_area_from_color_lines(BUFFER_TYPE) { \
+	int x, y, src, dst = 0; \
+	BUFFER_TYPE *inbuf = (BUFFER_TYPE *)read_buffer; \
+	BUFFER_TYPE *out = (BUFFER_TYPE *)outbuf; \
+	int color_offset; \
+	if (ser_file->color_id == SER_BGR) { \
+		color_offset = 2 - layer; \
+	} else { \
+		color_offset = layer; \
+	} \
+	for (y = 0; y < area->h; y++) { \
+		src = (y * ser_file->image_width + area->x) * 3 + color_offset; \
+		for (x = 0; x < area->w; x++) { \
+			out[dst++] = inbuf[src]; \
+			src += 3; \
+		} \
+	} \
+}
+
+/* reading an area from a SER frame, for one layer only, either layer == -1 for
+ * monochrome and debayer, or 0-2 for color.
+ * the area is read in one read(2) call to limit the number of syscalls, for a
+ * full-width area of same height as requested, then cropped horizontally to
+ * get the requested area.
+ * This function is the first one of siril to handle two different data types
+ * (BYTE and WORD) for the same algorithm! This uses VIPS-style macros.
+ * */
+static int read_area_from_image(struct ser_struct *ser_file, const int frame_no,
+		WORD *outbuf, const rectangle *area, const int layer) {
+	int64_t offset, frame_size;
+	int retval = 0;
+	WORD *read_buffer;
+	size_t read_size = ser_file->image_width * area->h * ser_file->byte_pixel_depth;
+	if (layer != -1) read_size *= 3;
+	if (layer != -1 || area->w != ser_file->image_width) {
+		// allocated space is probably not enough to
+		// store whole lines or RGB data
+		read_buffer = malloc(read_size);
+	}
+	else read_buffer = outbuf;
+
+	frame_size = ser_file->image_width * ser_file->image_height *
+		ser_file->number_of_planes * ser_file->byte_pixel_depth;
+
+#ifdef _OPENMP
+	omp_set_lock(&ser_file->fd_lock);
+#endif
+	// we read the full-stride rectangle that contains the requested area
+	offset = SER_HEADER_LEN + frame_size * frame_no +	// requested frame
+		area->y * ser_file->image_width *
+		ser_file->byte_pixel_depth * (layer != -1 ? 3 : 1);	// requested area
+
+	if ((int64_t)-1 == lseek64(ser_file->fd, offset, SEEK_SET)) {
+		perror("lseek in SER");
+		retval = -1;
+	} else {
+		if (read(ser_file->fd, read_buffer, read_size) != read_size)
+			retval = -1;
+	}
+#ifdef _OPENMP
+	omp_unset_lock(&ser_file->fd_lock);
+#endif
+	if (!retval) {
+		if (area->w != ser_file->image_width) {
+			// here we crop x-wise our area
+			if (layer != -1) {
+				/* reorder the RGBRGB to RRGGBB and crop */
+				if (ser_file->byte_pixel_depth == SER_PIXEL_DEPTH_8) {
+					crop_area_from_color_lines(BYTE);
+				} else if (ser_file->byte_pixel_depth == SER_PIXEL_DEPTH_16) {
+					crop_area_from_color_lines(WORD);
+				}
+			} else {
+				/* just crop */
+				if (ser_file->byte_pixel_depth == SER_PIXEL_DEPTH_8) {
+					crop_area_from_lines(BYTE);
+				} else if (ser_file->byte_pixel_depth == SER_PIXEL_DEPTH_16) {
+					crop_area_from_lines(WORD);
+				}
+			}
+		} else if (layer != -1) {
+			/* just reorder RGB data, the crop function works too */
+			if (ser_file->byte_pixel_depth == SER_PIXEL_DEPTH_8) {
+				crop_area_from_color_lines(BYTE);
+			} else if (ser_file->byte_pixel_depth == SER_PIXEL_DEPTH_16) {
+				crop_area_from_color_lines(WORD);
+			}
+		}
+	}
+	if (layer != -1 || area->w != ser_file->image_width)
+		free(read_buffer);
+	return retval;
+}
+
 /* read an area of an image in an opened SER sequence */
 int ser_read_opened_partial(struct ser_struct *ser_file, int layer,
 		int frame_no, WORD *buffer, const rectangle *area) {
-	int64_t offset;
-	int frame_size, read_size, retval, xoffset, yoffset, x, y, color_offset;
+	int xoffset, yoffset, x, y;
 	ser_color type_ser;
-	WORD *rawbuf, *demosaiced_buf, *rgb_buf;
+	WORD *rawbuf, *demosaiced_buf;
 	rectangle debayer_area, image_area;
 	sensor_pattern sensortmp;
 
 	if (!ser_file || ser_file->fd <= 0 || frame_no < 0
 			|| frame_no >= ser_file->frame_count)
 		return -1;
-	frame_size = ser_file->image_width * ser_file->image_height *
-		ser_file->number_of_planes * ser_file->byte_pixel_depth;
 
-	/* If the user checks the SER CFA box, the video is opened in B&W
-	 * RGB and BGR are not coming from raw data. In consequence CFA does
-	 * not exist for these kind of cam */
 	type_ser = ser_file->color_id;
-	if (!com.debayer.open_debayer && type_ser != SER_RGB && type_ser !=
-			SER_BGR)
+	if (!com.debayer.open_debayer &&
+			type_ser != SER_RGB && type_ser != SER_BGR)
 		type_ser = SER_MONO;
 
 	switch (type_ser) {
 	case SER_MONO:
-		offset = SER_HEADER_LEN + (int64_t)frame_size * (int64_t)frame_no +	// requested frame
-			(int64_t)(area->y * ser_file->image_width + area->x)
-						* ser_file->byte_pixel_depth;	// requested area
-#ifdef _OPENMP
-		omp_set_lock(&ser_file->fd_lock);
-#endif
-		if ((int64_t) -1 == lseek64(ser_file->fd, offset, SEEK_SET)) {
-#ifdef _OPENMP
-			omp_unset_lock(&ser_file->fd_lock);
-#endif
+		if (read_area_from_image(ser_file, frame_no, buffer, area, -1))
 			return -1;
-		}
-		read_size = area->w * area->h * ser_file->byte_pixel_depth;
-		retval = read(ser_file->fd, buffer, read_size);
-#ifdef _OPENMP
-		omp_unset_lock(&ser_file->fd_lock);
-#endif
-		if (retval != read_size)
-			return -1;
-
 		ser_manage_endianess_and_depth(ser_file, buffer, area->w * area->h);
 		break;
 
@@ -830,7 +945,7 @@ int ser_read_opened_partial(struct ser_struct *ser_file, int layer,
 		sensortmp = com.debayer.bayer_pattern;
 		if (com.debayer.use_bayer_header) {
 			sensor_pattern bayer;
-			bayer = retrieveSERBayerPattern(type_ser);
+			bayer = get_SER_Bayer_Pattern(type_ser);
 			if (bayer != com.debayer.bayer_pattern) {
 				if (bayer == BAYER_FILTER_NONE && warning == FALSE) {
 					siril_log_color_message(_("No Bayer pattern found in the header file.\n"), "red");
@@ -855,39 +970,14 @@ int ser_read_opened_partial(struct ser_struct *ser_file, int layer,
 			.w = ser_file->image_width, .h = ser_file->image_height };
 		get_debayer_area(area, &debayer_area, &image_area, &xoffset, &yoffset);
 
-		offset = SER_HEADER_LEN + frame_size * frame_no +	// requested frame
-				(debayer_area.y * ser_file->image_width + debayer_area.x)
-						* ser_file->byte_pixel_depth;	// requested area
-#ifdef _OPENMP
-		omp_set_lock(&ser_file->fd_lock);
-#endif
-		if ((int64_t) -1 == lseek64(ser_file->fd, offset, SEEK_SET)) {
-#ifdef _OPENMP
-			omp_unset_lock(&ser_file->fd_lock);
-#endif
-			return -1;
-		}
-
 		// allocating a buffer for WORD because it's going to be converted in-place
 		rawbuf = malloc(debayer_area.w * debayer_area.h * sizeof(WORD));
 		if (!rawbuf) {
-#ifdef _OPENMP
-			omp_unset_lock(&ser_file->fd_lock);
-#endif
 			siril_log_message(_("Out of memory - aborting\n"));
 			return -1;
 		}
-		read_size = debayer_area.w * debayer_area.h * ser_file->byte_pixel_depth;
-		retval = read(ser_file->fd, rawbuf, read_size);
-#ifdef _OPENMP
-		omp_unset_lock(&ser_file->fd_lock);
-#endif
-		if (retval != read_size) {
-			free(rawbuf);
-			perror("read");
+		if (read_area_from_image(ser_file, frame_no, rawbuf, &debayer_area, -1))
 			return -1;
-		}
-
 		ser_manage_endianess_and_depth(ser_file, rawbuf, debayer_area.w * debayer_area.h);
 
 		demosaiced_buf = debayer_buffer(rawbuf, &debayer_area.w,
@@ -911,56 +1001,13 @@ int ser_read_opened_partial(struct ser_struct *ser_file, int layer,
 		free(demosaiced_buf);
 		com.debayer.bayer_pattern = sensortmp;
 		break;
+
 	case SER_BGR:
 	case SER_RGB:
 		assert(ser_file->number_of_planes == 3);
-
-		offset = SER_HEADER_LEN + frame_size * frame_no +	// requested frame
-			(area->y * ser_file->image_width + area->x) *
-			ser_file->byte_pixel_depth * 3;	// requested area
-#ifdef _OPENMP
-		omp_set_lock(&ser_file->fd_lock);
-#endif
-		if ((int64_t) -1 == lseek64(ser_file->fd, offset, SEEK_SET)) {
-#ifdef _OPENMP
-			omp_unset_lock(&ser_file->fd_lock);
-#endif
+		if (read_area_from_image(ser_file, frame_no, buffer, area, layer))
 			return -1;
-		}
-
-		read_size = area->w * area->h * ser_file->byte_pixel_depth * 3;
-		// allocating a buffer for WORD because it's going to be converted in-place
-		rgb_buf = malloc(area->w * area->h * 3 * sizeof(WORD));
-		if (!rgb_buf) {
-#ifdef _OPENMP
-			omp_unset_lock(&ser_file->fd_lock);
-#endif
-			siril_log_message(_("Out of memory - aborting\n"));
-			return -1;
-		}
-		retval = read(ser_file->fd, rgb_buf, read_size);
-#ifdef _OPENMP
-		omp_unset_lock(&ser_file->fd_lock);
-#endif
-		if (retval != read_size) {
-			free(rgb_buf);
-			perror("read");
-			return -1;
-		}
-
-		ser_manage_endianess_and_depth(ser_file, rgb_buf, area->w * area->h * 3);
-
-		color_offset = layer;
-		if (type_ser == SER_BGR) {
-			color_offset = 2 - layer;
-		}
-
-		for (y = 0; y < area->h; y++) {
-			for (x = 0; x < area->w; x++) {
-				buffer[y*area->w + x] = rgb_buf[y*area->w*3 + x*3 + color_offset]; 
-			}
-		}
-		free(rgb_buf);
+		ser_manage_endianess_and_depth(ser_file, buffer, area->w * area->h);
 		break;
 	default:
 		siril_log_message(_("This type of Bayer pattern is not handled yet.\n"));
@@ -970,10 +1017,17 @@ int ser_read_opened_partial(struct ser_struct *ser_file, int layer,
 	return 0;
 }
 
+int ser_read_opened_partial_fits(struct ser_struct *ser_file, int layer,
+		int frame_no, fits *fit, const rectangle *area) {
+	if (new_fit_image(&fit, area->w, area->h, 1))
+		return -1;
+	return ser_read_opened_partial(ser_file, layer, frame_no, fit->pdata[0], area);
+}
+
 int ser_write_frame_from_fit(struct ser_struct *ser_file, fits *fit, int frame_no) {
-	int frame_size, pixel, plane, dest;
+	int pixel, plane, dest;
 	int ret, retval = 0;
-	int64_t offset;
+	int64_t offset, frame_size;
 	BYTE *data8 = NULL;			// for 8-bit files
 	WORD *data16 = NULL;		// for 16-bit files
 
@@ -992,13 +1046,16 @@ int ser_write_frame_from_fit(struct ser_struct *ser_file, fits *fit, int frame_n
 	frame_size = ser_file->image_width * ser_file->image_height *
 		ser_file->number_of_planes;
 
-	offset = SER_HEADER_LEN	+ (int64_t) frame_size *
-			(int64_t) ser_file->byte_pixel_depth * (int64_t) frame_no;
+	offset = SER_HEADER_LEN	+ frame_size *
+			(int64_t)ser_file->byte_pixel_depth * (int64_t)frame_no;
 
-	if (ser_file->byte_pixel_depth == SER_PIXEL_DEPTH_8)
+	if (ser_file->byte_pixel_depth == SER_PIXEL_DEPTH_8) {
 		data8 = malloc(frame_size * ser_file->byte_pixel_depth);
-	else
+		if (!data8) return -1;
+	} else {
 		data16 = malloc(frame_size * ser_file->byte_pixel_depth);
+		if (!data16) return -1;
+	}
 
 	for (plane = 0; plane < ser_file->number_of_planes; plane++) {
 		dest = plane;
@@ -1054,7 +1111,12 @@ int ser_write_frame_from_fit(struct ser_struct *ser_file, fits *fit, int frame_n
 #pragma omp atomic
 #endif
 	ser_file->frame_count++;
-	retval = 0;
+
+	if (!ser_alloc_ts(ser_file, frame_no)) {
+		uint64_t utc, local;
+		FITS_date_key_to_Unix_time(fit->date_obs, &utc, &local);
+		ser_file->ts[frame_no] = utc;
+	}
 
 free_and_quit:
 	if (data8) free(data8);
