@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2017 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2018 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -31,14 +31,16 @@
 #include "io/single_image.h"
 #include "gui/PSF_list.h"
 #include "gui/histogram.h"
+#include "gui/progress_and_log.h"
 #include "core/undo.h"
 
 /* Closes and frees resources attached to the single image opened in gfit.
  * If a sequence is loaded and one of its images is displayed, nothing is done.
  */
 void close_single_image() {
-	if (sequence_is_loaded() && com.seq.current != RESULT_IMAGE)
+	if (sequence_is_loaded() && com.seq.current >= 0)
 		return;
+	memset(&com.selection, 0, sizeof(rectangle));
 	free_image_data();
 	undo_flush();
 }
@@ -159,12 +161,15 @@ int open_single_image(const char* filename) {
 		return 1;
 	}
 
+	// Closing a sequence
 	if (sequence_is_loaded()) {
 		char *basename = g_path_get_basename(com.seq.seqname);
+		// TODO: is basename useful here?
 		siril_log_message(_("Closing sequence %s\n"), basename);
 		g_free(basename);
 		clear_sequence_list();
 		free_sequence(&(com.seq), FALSE);
+		clear_stars_list();
 		initialize_sequence(&com.seq, FALSE);
 	}
 	fprintf(stdout, "Loading image OK, now displaying\n");
@@ -217,6 +222,7 @@ void open_single_image_from_gfit(char *realname) {
 
 /* searches the image for minimum and maximum pixel value, on each layer
  * the values are stored in fit->min[layer] and fit->max[layer] */
+//TODO: TO REWRITE WITH REDUCE
 int image_find_minmax(fits *fit, int force_minmax){
 	int i, layer;
 
