@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2017 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2018 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -34,6 +34,7 @@
 #include "core/siril.h"
 #include "gui/callbacks.h"
 #include "gui/plot.h"
+#include "gui/progress_and_log.h"
 #include "core/proto.h"
 #include "core/initfile.h"
 #include "registration/registration.h"
@@ -47,10 +48,8 @@
 #include "algos/quality.h"
 #include "io/sequence.h"
 #include "io/ser.h"
-#ifdef HAVE_OPENCV
 #include "opencv/opencv.h"
 #include "opencv/ecc/ecc.h"
-#endif
 
 #define MAX_STARS_FITTED 2000
 #undef DEBUG
@@ -59,23 +58,18 @@ static char *tooltip_text[] = { N_("One Star Registration: This is the simplest
 		"Because only one star is concerned for register, images are aligned using shifting "
 		"(at a fraction of pixel). No rotation or scaling are performed. "
 		"Shifts at pixel precision are saved in seq file."),
-#ifdef HAVE_OPENCV
 		N_("Global Star Alignment: This is a more powerful and accurate algorithm (but also slower) "
 		"to perform deep-sky images. The global matching is based on triangle similarity method for automatically "
 		"identify common stars in each image. "
 		"A new sequence is created with the prefix of your choice (r_ by default)."),
-#endif
 		N_("Image Pattern Alignment: This is a simple registration by translation method "
 		"using cross correlation in the spatial domain. This method is fast and is used to register "
 		"planetary movies. It can also be used for some deep-sky images registration. "
-		"Shifts at pixel precision are saved in seq file.")
-#ifdef HAVE_OPENCV
-,
+		"Shifts at pixel precision are saved in seq file."),
 		N_("Enhanced Correlation Coefficient Maximization: It is based on the enhanced correlation "
 		"coefficient maximization algorithm. This method is more complex and slower than Image Pattern Alignment "
 		"but no selection is required. It is good for moon surface images registration. Only translation is taken "
 		"into account yet.")
-#endif
 };
 /* callback for the selected area event */
 void _reg_selected_area_callback() {
@@ -104,16 +98,12 @@ void initialize_registration_methods() {
 
 	reg_methods[i++] = new_reg_method(_("One Star Registration (deep-sky)"),
 			&register_shift_fwhm, REQUIRES_ANY_SELECTION, REGTYPE_DEEPSKY);
-#ifdef HAVE_OPENCV
 	reg_methods[i++] = new_reg_method(_("Global Star Alignment (deep-sky)"),
 			&register_star_alignment, REQUIRES_NO_SELECTION, REGTYPE_DEEPSKY);
-#endif
 	reg_methods[i++] = new_reg_method(_("Image Pattern Alignment (planetary - full disk)"),
 			&register_shift_dft, REQUIRES_SQUARED_SELECTION, REGTYPE_PLANETARY);
-#ifdef HAVE_OPENCV
 	reg_methods[i++] = new_reg_method(_("Enhanced Correlation Coefficient (planetary - surfaces)"),
 			&register_ecc, REQUIRES_NO_SELECTION, REGTYPE_PLANETARY);
-#endif
 	reg_methods[i] = NULL;
 
 	tip = g_string_new ("");
@@ -162,16 +152,20 @@ struct registration_method *get_selected_registration_method() {
 
 static void normalizeQualityData(struct registration_args *args, double q_min, double q_max) {
 	int frame;
+	double diff = q_max - q_min;
+
+	/* this case occurs when all images but one are excluded */
+	if (diff == 0) {
+		q_min = 0;
+		diff = q_max;
+	}
 
 	for (frame = 0; frame < args->seq->number; ++frame) {
-		if (args->run_in_thread && !get_thread_run()) {
-			break;
-		}
-		if (!args->process_all_frames && !args->seq->imgparam[frame].incl)
-			continue;
-
 		args->seq->regparam[args->layer][frame].quality -= q_min;
-		args->seq->regparam[args->layer][frame].quality /= (q_max - q_min);
+		args->seq->regparam[args->layer][frame].quality /= diff;
+		/* if thread has been manually stopped, some values will be < 0 */
+		if (args->seq->regparam[args->layer][frame].quality < 0)
+			args->seq->regparam[args->layer][frame].quality = -1.0;
 	}
 }
 
@@ -190,15 +184,12 @@ int register_shift_dft(struct registration_args *args) {
 	float nb_frames, cur_nb;
 	int ref_image;
 	regdata *current_regdata;
-	rectangle full_area;	// the area to use after getting image_part
 	double q_max = 0, q_min = DBL_MAX;
 	int q_index = -1;
 
 	/* the selection needs to be squared for the DFT */
 	assert(args->selection.w == args->selection.h);
 	size = args->selection.w;
-	full_area.x = full_area.y = 0;
-	full_area.h = full_area.w = size;
 	sqsize = size * size;
 
 	if (args->process_all_frames)
@@ -223,11 +214,8 @@ int register_shift_dft(struct registration_args *args) {
 	}
 
 	/* loading reference frame */
-	if (args->seq->reference_image == -1)
-		ref_image = 0;
-	else
-		ref_image = args->seq->reference_image;
-
+	ref_image = sequence_find_refimage(args->seq);
+	
 	set_progress_bar_data(
 			_("Register DFT: loading and processing reference frame"),
 			PROGRESS_NONE);
@@ -264,8 +252,8 @@ int register_shift_dft(struct registration_args *args) {
 	current_regdata[ref_image].quality = QualityEstimate(&fit_ref, args->layer, QUALTYPE_NORMAL);
 	clearfits(&fit_ref);
 	fftw_execute_dft(p, ref, in); /* repeat as needed */
-	current_regdata[ref_image].shiftx = 0;
-	current_regdata[ref_image].shifty = 0;
+	current_regdata[ref_image].shiftx = 0.0;
+	current_regdata[ref_image].shifty = 0.0;
 
 	q_min = q_max = current_regdata[ref_image].quality;
 	q_index = ref_image;
@@ -350,8 +338,11 @@ int register_shift_dft(struct registration_args *args) {
 					shiftx -= size;
 				}
 
-				current_regdata[frame].shiftx = shiftx;
-				current_regdata[frame].shifty = shifty;
+				/* for Y, it's a bit special because FITS are upside-down */
+				float sign = args->seq->type == SEQ_SER ? -1 : 1;
+
+				current_regdata[frame].shiftx = (float) shiftx;
+				current_regdata[frame].shifty = (float) shifty * sign;
 
 				/* shiftx and shifty are the x and y values for translation that
 				 * would make this image aligned with the reference image.
@@ -360,7 +351,7 @@ int register_shift_dft(struct registration_args *args) {
 				 */
 #ifdef DEBUG
 				fprintf(stderr,
-						"reg: frame %d, shiftx=%d shifty=%d quality=%g\n",
+						"reg: frame %d, shiftx=%f shifty=%f quality=%g\n",
 						args->seq->imgparam[frame].filenum,
 						current_regdata[frame].shiftx, current_regdata[frame].shifty,
 						current_regdata[frame].quality);
@@ -391,6 +382,10 @@ int register_shift_dft(struct registration_args *args) {
 	fftw_free(convol);
 	if (!ret) {
 		args->seq->regparam[args->layer] = current_regdata;
+		if (args->x2upscale)
+			args->seq->upscale_at_stacking = 2.0;
+		else
+			args->seq->upscale_at_stacking = 1.0;
 		normalizeQualityData(args, q_min, q_max);
 		update_used_memory();
 		siril_log_message(_("Registration finished.\n"));
@@ -429,10 +424,7 @@ int register_shift_fwhm(struct registration_args *args) {
 		nb_frames = (float) args->seq->selnum;
 
 	/* loading reference frame */
-	if (args->seq->reference_image == -1)
-		ref_image = 0;
-	else
-		ref_image = args->seq->reference_image;
+	ref_image = sequence_find_refimage(args->seq);
 	if (!current_regdata[ref_image].fwhm_data) {
 		siril_log_message(
 				_("Registration PSF: failed to compute PSF for reference frame at least\n"));
@@ -455,8 +447,8 @@ int register_shift_fwhm(struct registration_args *args) {
 		if (!args->process_all_frames && !args->seq->imgparam[frame].incl)
 			continue;
 		if (frame == ref_image || !current_regdata[frame].fwhm_data) {
-			current_regdata[frame].shiftx = 0;
-			current_regdata[frame].shifty = 0;
+			current_regdata[frame].shiftx = 0.0;
+			current_regdata[frame].shifty = 0.0;
 			continue;
 		}
 		if (current_regdata[frame].fwhm < fwhm_min
@@ -465,16 +457,16 @@ int register_shift_fwhm(struct registration_args *args) {
 			fwhm_index = frame;
 		}
 		tmp = reference_xpos - current_regdata[frame].fwhm_data->xpos;
-		current_regdata[frame].shiftx = round_to_int(tmp);
+		current_regdata[frame].shiftx = tmp;
 		tmp = current_regdata[frame].fwhm_data->ypos - reference_ypos;
-		current_regdata[frame].shifty = round_to_int(tmp);
+		current_regdata[frame].shifty = tmp;
 
 		/* shiftx and shifty are the x and y values for translation that
 		 * would make this image aligned with the reference image.
 		 * WARNING: the y value is counted backwards, since the FITS is
 		 * stored down from up.
 		 */
-		fprintf(stderr, "reg: file %d, shiftx=%d shifty=%d\n",
+		fprintf(stderr, "reg: file %d, shiftx=%f shifty=%f\n",
 				args->seq->imgparam[frame].filenum,
 				current_regdata[frame].shiftx, current_regdata[frame].shifty);
 		cur_nb += 1.f;
@@ -482,6 +474,10 @@ int register_shift_fwhm(struct registration_args *args) {
 	}
 
 	args->seq->regparam[args->layer] = current_regdata;
+	if (args->x2upscale)
+		args->seq->upscale_at_stacking = 2.0;
+	else
+		args->seq->upscale_at_stacking = 1.0;
 	update_used_memory();
 	siril_log_message(_("Registration finished.\n"));
 	siril_log_color_message(_("Best frame: #%d with fwhm=%.3g.\n"), "bold",
@@ -489,8 +485,6 @@ int register_shift_fwhm(struct registration_args *args) {
 	return 0;
 }
 
-#ifdef HAVE_OPENCV
-
 static void _print_result(Homography *H, float FWHMx, float FWHMy) {
 	double rotation, scale, scaleX, scaleY;
 	point shift;
@@ -564,9 +558,7 @@ int register_star_alignment(struct registration_args *args) {
 		nb_frames = (float) args->seq->selnum;
 
 	/* loading reference frame */
-	if (args->seq->reference_image == -1)
-		ref_image = 0;
-	else ref_image = args->seq->reference_image;
+	ref_image = sequence_find_refimage(args->seq);
 
 	/* first we're looking for stars in reference image */
 	ret = seq_read_frame(args->seq, ref_image, &fit);
@@ -579,8 +571,8 @@ int register_star_alignment(struct registration_args *args) {
 	}
 	siril_log_color_message(_("Reference Image:\n"), "green");
 
-	if ((com.selection.w != 0) && (com.selection.h != 0) && args->matchSelection) {
-		com.stars = peaker(&fit, args->layer, &sf, &com.selection);
+	if ((args->selection.w != 0) && (args->selection.h != 0) && args->matchSelection) {
+		com.stars = peaker(&fit, args->layer, &sf, &args->selection);
 	}
 	else {
 		com.stars = peaker(&fit, args->layer, &sf, NULL);
@@ -598,6 +590,19 @@ int register_star_alignment(struct registration_args *args) {
 
 	ref.x = fit.rx;
 	ref.y = fit.ry;
+	if (args->x2upscale) {
+		if (args->translation_only) {
+			args->seq->upscale_at_stacking = 2.0;
+		} else {
+			ref.x *= 2.0;
+			ref.y *= 2.0;
+		}
+	}
+	else {
+		if (args->translation_only) {
+			args->seq->upscale_at_stacking = 1.0;
+		}
+	}
 
 	/* we copy com.stars to refstars in case user take a look to another image of the sequence
 	 * that would destroy com.stars
@@ -627,8 +632,12 @@ int register_star_alignment(struct registration_args *args) {
 	if (args->process_all_frames)
 		args->new_total = args->seq->number;
 	else args->new_total = args->seq->selnum;
-	args->imgparam = calloc(args->new_total, sizeof(imgdata));
-	args->regparam = calloc(args->new_total, sizeof(regdata));
+
+	if (!args->translation_only) {
+		// allocate destination sequence data
+		args->imgparam = calloc(args->new_total, sizeof(imgdata));
+		args->regparam = calloc(args->new_total, sizeof(regdata));
+	}
 
 	if (args->seq->type == SEQ_SER) {
 		char dest[256];
@@ -718,6 +727,11 @@ int register_star_alignment(struct registration_args *args) {
 
 					if (!args->translation_only) {
 						fits_flip_top_to_bottom(&fit);	// this is because in cvTransformImage, rotation center point is at (0, 0)
+						if (args->x2upscale) {
+							cvResizeGaussian(&fit, fit.rx * 2, fit.ry * 2, OPENCV_NEAREST);
+							H.h02 *= 2.0;
+							H.h12 *= 2.0;
+						}
 						cvTransformImage(&fit, ref, H, args->interpolation);
 						fits_flip_top_to_bottom(&fit);
 					}
@@ -727,6 +741,11 @@ int register_star_alignment(struct registration_args *args) {
 						free(stars[i++]);
 					free(stars);
 				}
+				else {
+					if (args->x2upscale && !args->translation_only) {
+						cvResizeGaussian(&fit, fit.rx * 2, fit.ry * 2, OPENCV_NEAREST);
+					}
+				}
 
 				if (!args->translation_only) {
 					if (args->seq->type == SEQ_SER) {
@@ -814,10 +833,7 @@ int register_ecc(struct registration_args *args) {
 		nb_frames = (float) args->seq->selnum;
 
 	/* loading reference frame */
-	if (args->seq->reference_image == -1)
-		ref_image = 0;
-	else
-		ref_image = args->seq->reference_image;
+	ref_image = sequence_find_refimage(args->seq);
 
 	memset(&ref, 0, sizeof(fits));
 
@@ -858,8 +874,8 @@ int register_ecc(struct registration_args *args) {
 			}
 			if (!args->process_all_frames && !args->seq->imgparam[frame].incl)
 				continue;
-			current_regdata[frame].shiftx = 0;
-			current_regdata[frame].shifty = 0;
+			current_regdata[frame].shiftx = 0.0;
+			current_regdata[frame].shifty = 0.0;
 
 			char tmpmsg[1024], tmpfilename[256];
 
@@ -880,7 +896,9 @@ int register_ecc(struct registration_args *args) {
 								_("Cannot perform ECC alignment for frame %d\n"),
 								frame);
 						/* We exclude this frame */
-						com.seq.imgparam[frame].incl = FALSE;
+						args->seq->imgparam[frame].incl = FALSE;
+						current_regdata[frame].quality = 0.0;
+						args->seq->selnum--;
 #ifdef _OPENMP
 #pragma omp atomic
 #endif
@@ -904,8 +922,8 @@ int register_ecc(struct registration_args *args) {
 						q_min = min(q_min, qual);
 					}
 
-					current_regdata[frame].shiftx = -round_to_int(reg_param.dx);
-					current_regdata[frame].shifty = -round_to_int(reg_param.dy);
+					current_regdata[frame].shiftx = -reg_param.dx;
+					current_regdata[frame].shifty = -reg_param.dy;
 
 #ifdef _OPENMP
 #pragma omp atomic
@@ -918,6 +936,11 @@ int register_ecc(struct registration_args *args) {
 		}
 	}
 	args->seq->regparam[args->layer] = current_regdata;
+	if (args->x2upscale)
+		args->seq->upscale_at_stacking = 2.0;
+	else
+		args->seq->upscale_at_stacking = 1.0;
+
 	normalizeQualityData(args, q_min, q_max);
 	clearfits(&ref);
 	update_used_memory();
@@ -929,8 +952,6 @@ int register_ecc(struct registration_args *args) {
 	return 0;
 }
 
-#endif
-
 void on_comboboxregmethod_changed(GtkComboBox *box, gpointer user_data) {
 	int index = 0;
 	gchar *text = gtk_combo_box_text_get_active_text (GTK_COMBO_BOX_TEXT(box));
@@ -995,13 +1016,17 @@ void update_reg_interface(gboolean dont_change_reg_radio) {
 		nb_images_reg = com.seq.number;
 	else
 		nb_images_reg = com.seq.selnum;
-	if (method && ((nb_images_reg > 1 && com.selection.w > 0 && com.selection.h > 0)
+
+	if (method && method->sel != REQUIRES_NO_SELECTION && com.seq.current == SCALED_IMAGE) {
+		gtk_widget_set_sensitive(go_register, FALSE);
+		gtk_widget_set_visible(newSequence, FALSE);
+		gtk_label_set_text(labelreginfo, _("Load a sequence first."));
+	}
+	else if (method && ((nb_images_reg > 1 && com.selection.w > 0 && com.selection.h > 0)
 			|| (nb_images_reg > 1 && method->sel == REQUIRES_NO_SELECTION))) {
 		gtk_widget_set_sensitive(go_register, TRUE);
 		gtk_label_set_text(labelreginfo, "");
-#ifdef HAVE_OPENCV
 		gtk_widget_set_visible(newSequence, method->method_ptr == &register_star_alignment);
-#endif
 		gtk_widget_set_visible(follow, method->method_ptr == &register_shift_fwhm);
 	} else {
 		gtk_widget_set_sensitive(go_register, FALSE);
@@ -1081,8 +1106,8 @@ void get_the_registration_area(struct registration_args *reg_args,
 		struct registration_method *method) {
 	int max;
 	switch (method->sel) {
+	/* even in the case of REQUIRES_NO_SELECTION selection is needed for MatchSelection of starAlignment */
 	case REQUIRES_NO_SELECTION:
-		break;
 	case REQUIRES_ANY_SELECTION:
 		memcpy(&reg_args->selection, &com.selection, sizeof(rectangle));
 		break;
@@ -1115,7 +1140,7 @@ void on_seqregister_button_clicked(GtkButton *button, gpointer user_data) {
 	struct registration_args *reg_args;
 	struct registration_method *method;
 	char *msg;
-	GtkToggleButton *regall, *follow, *matchSel, *no_translate;
+	GtkToggleButton *regall, *follow, *matchSel, *no_translate, *x2upscale;
 	GtkComboBox *cbbt_layers;
 	GtkComboBoxText *ComboBoxRegInter;
 
@@ -1137,31 +1162,45 @@ void on_seqregister_button_clicked(GtkButton *button, gpointer user_data) {
 	}
 	// TODO: check for reentrance
 
-	reg_args = malloc(sizeof(struct registration_args));
+	reg_args = calloc(1, sizeof(struct registration_args));
 
 	control_window_switch_to_tab(OUTPUT_LOGS);
 
 	/* filling the arguments for registration */
-	reg_args->seq = &com.seq;
 	regall = GTK_TOGGLE_BUTTON(lookup_widget("regallbutton"));
 	follow = GTK_TOGGLE_BUTTON(lookup_widget("followStarCheckButton"));
 	matchSel = GTK_TOGGLE_BUTTON(lookup_widget("checkStarSelect"));
 	no_translate = GTK_TOGGLE_BUTTON(lookup_widget("regTranslationOnly"));
+	x2upscale = GTK_TOGGLE_BUTTON(lookup_widget("upscaleCheckButton"));
+	cbbt_layers = GTK_COMBO_BOX(
+			gtk_builder_get_object(builder, "comboboxreglayer"));
+	ComboBoxRegInter = GTK_COMBO_BOX_TEXT(
+			gtk_builder_get_object(builder, "ComboBoxRegInter"));
+
+	reg_args->func = method->method_ptr;
+	reg_args->seq = &com.seq;
 	reg_args->process_all_frames = gtk_toggle_button_get_active(regall);
 	reg_args->follow_star = gtk_toggle_button_get_active(follow);
 	reg_args->matchSelection = gtk_toggle_button_get_active(matchSel);
 	reg_args->translation_only = gtk_toggle_button_get_active(no_translate);
+	reg_args->x2upscale = gtk_toggle_button_get_active(x2upscale);
+	/* Here we should test available free disk space for Drizzle operation */
+	if (reg_args->x2upscale) {
+		double size = seq_compute_size(reg_args->seq);
+		double diff = test_available_space(size * 4.0); //FIXME: 4 is only ok for x2 Drizzle
+		if (diff < 0.0) {
+			msg = siril_log_message(_("Not enough disk space to perform Drizzle operation !!\n"));
+			show_dialog(msg, _("Error"), "gtk-dialog-error");
+			free(reg_args);
+			return;
+		}
+	}
 	/* getting the selected registration layer from the combo box. The value is the index
 	 * of the selected line, and they are in the same order than layers so there should be
 	 * an exact matching between the two */
-	cbbt_layers = GTK_COMBO_BOX(
-			gtk_builder_get_object(builder, "comboboxreglayer"));
 	reg_args->layer = gtk_combo_box_get_active(cbbt_layers);
-	ComboBoxRegInter = GTK_COMBO_BOX_TEXT(
-			gtk_builder_get_object(builder, "ComboBoxRegInter"));
 	reg_args->interpolation = gtk_combo_box_get_active(GTK_COMBO_BOX(ComboBoxRegInter));
-	get_the_registration_area(reg_args, method);
-	reg_args->func = method->method_ptr;
+	get_the_registration_area(reg_args, method);	// sets selection
 	reg_args->run_in_thread = TRUE;
 	reg_args->prefix = gtk_entry_get_text(
 			GTK_ENTRY(gtk_builder_get_object(builder, "regseqname_entry")));
@@ -1195,7 +1234,6 @@ static gboolean end_register_idle(gpointer p) {
 		set_layers_for_registration();	// update display of available reg data
 
 		/* Load new sequence. Only star alignment method uses new sequence. */
-#ifdef HAVE_OPENCV
 		if (args->func == &register_star_alignment) {
 			if (args->load_new_sequence) {
 				sequence *seq;
@@ -1212,7 +1250,7 @@ static gboolean end_register_idle(gpointer p) {
 
 				sprintf(rseqname, "%s%s.seq", args->prefix, seqname);
 				g_free(seqname);
-				unlink(rseqname);	// remove previous to overwrite
+				g_unlink(rseqname);	// remove previous to overwrite
 				char *newname = remove_ext_from_filename(rseqname);
 				seq->seqname = newname;
 				seq->number = args->new_total;
@@ -1244,14 +1282,12 @@ static gboolean end_register_idle(gpointer p) {
 			}
 			clear_stars_list();
 		}
-#endif
 	}
 	set_progress_bar_data(_("Registration complete."), PROGRESS_DONE);
 	drawPlot();
-#ifdef HAVE_OPENCV
 failed_end:
-#endif
-	update_stack_interface();
+	update_stack_interface(TRUE);
+	adjust_sellabel();
 	update_used_memory();
 	set_cursor_waiting(FALSE);
 #ifdef MAC_INTEGRATION
